# Copyright 2012 OpenStack Foundation
# Copyright 2013 Hewlett-Packard Development Company, L.P.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import collections
import re

from oslo_log import log as logging
import testtools

from tempest.common import utils
from tempest.common import waiters
from tempest import config
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators
from tempest.lib import exceptions
from tempest.scenario import manager

CONF = config.CONF
LOG = logging.getLogger(__name__)

Floating_IP_tuple = collections.namedtuple('Floating_IP_tuple',
                                           ['floating_ip', 'server'])


class TestNetworkBasicOps(manager.NetworkScenarioTest):

    """The test suite of network basic operations

    This smoke test suite assumes that Nova has been configured to
    boot VM's with Neutron-managed networking, and attempts to
    verify network connectivity as follows:

     There are presumed to be two types of networks: tenant and
     public.  A tenant network may or may not be reachable from the
     Tempest host.  A public network is assumed to be reachable from
     the Tempest host, and it should be possible to associate a public
     ('floating') IP address with a tenant ('fixed') IP address to
     facilitate external connectivity to a potentially unroutable
     tenant IP address.

     This test suite can be configured to test network connectivity to
     a VM via a tenant network, a public network, or both.  If both
     networking types are to be evaluated, tests that need to be
     executed remotely on the VM (via ssh) will only be run against
     one of the networks (to minimize test execution time).

     Determine which types of networks to test as follows:

     * Configure tenant network checks (via the
       'project_networks_reachable' key) if the Tempest host should
       have direct connectivity to tenant networks.  This is likely to
       be the case if Tempest is running on the same host as a
       single-node devstack installation with IP namespaces disabled.

     * Configure checks for a public network if a public network has
       been configured prior to the test suite being run and if the
       Tempest host should have connectivity to that public network.
       Checking connectivity for a public network requires that a
       value be provided for 'public_network_id'.  A value can
       optionally be provided for 'public_router_id' if tenants will
       use a shared router to access a public network (as is likely to
       be the case when IP namespaces are not enabled).  If a value is
       not provided for 'public_router_id', a router will be created
       for each tenant and use the network identified by
       'public_network_id' as its gateway.

    """

    @classmethod
    def skip_checks(cls):
        super(TestNetworkBasicOps, cls).skip_checks()
        if not (CONF.network.project_networks_reachable or
                CONF.network.public_network_id):
            msg = ('Either project_networks_reachable must be "true", or '
                   'public_network_id must be defined.')
            raise cls.skipException(msg)
        for ext in ['router', 'security-group']:
            if not utils.is_extension_enabled(ext, 'network'):
                msg = "%s extension not enabled." % ext
                raise cls.skipException(msg)
        if not CONF.network_feature_enabled.floating_ips:
            raise cls.skipException("Floating ips are not available")

    @classmethod
    def setup_credentials(cls):
        # Create no network resources for these tests.
        cls.set_network_resources()
        super(TestNetworkBasicOps, cls).setup_credentials()

    def setUp(self):
        super(TestNetworkBasicOps, self).setUp()
        self.keypairs = {}
        self.servers = []

    def _setup_network_and_servers(self, **kwargs):
        boot_with_port = kwargs.pop('boot_with_port', False)
        self.network, self.subnet, self.router = (
            self.setup_network_subnet_with_router(**kwargs))
        self.check_networks()

        self.ports = []
        port_id = None
        if boot_with_port:
            # create a port on the network and boot with that
            port_id = self.create_port(self.network['id'])['id']
            self.ports.append({'port': port_id})

        server = self._create_server(self.network, port_id)
        ssh_login = CONF.validation.image_ssh_user
        for server in self.servers:
            # call the common method in the parent class
            self.check_tenant_network_connectivity(
                server, ssh_login, self._get_server_key(server),
                servers_for_debug=self.servers)

        floating_ip = self.create_floating_ip(server)
        self.floating_ip_tuple = Floating_IP_tuple(floating_ip, server)

    def check_networks(self):
        """Checks that we see the newly created network/subnet/router

        via checking the result of list_[networks,routers,subnets]
        """

        seen_nets = self.os_admin.networks_client.list_networks()
        seen_names = [n['name'] for n in seen_nets['networks']]
        seen_ids = [n['id'] for n in seen_nets['networks']]
        self.assertIn(self.network['name'], seen_names)
        self.assertIn(self.network['id'], seen_ids)

        if self.subnet:
            seen_subnets = self.os_admin.subnets_client.list_subnets()
            seen_net_ids = [n['network_id'] for n in seen_subnets['subnets']]
            seen_subnet_ids = [n['id'] for n in seen_subnets['subnets']]
            self.assertIn(self.network['id'], seen_net_ids)
            self.assertIn(self.subnet['id'], seen_subnet_ids)

        if self.router:
            seen_routers = self.os_admin.routers_client.list_routers()
            seen_router_ids = [n['id'] for n in seen_routers['routers']]
            seen_router_names = [n['name'] for n in seen_routers['routers']]
            self.assertIn(self.router['name'],
                          seen_router_names)
            self.assertIn(self.router['id'],
                          seen_router_ids)

    def _create_server(self, network, port_id=None):
        keypair = self.create_keypair()
        self.keypairs[keypair['name']] = keypair
        security_groups = [
            {'name': self.create_security_group()['name']}
        ]
        network = {'uuid': network['id']}
        if port_id is not None:
            network['port'] = port_id

        server = self.create_server(
            networks=[network],
            key_name=keypair['name'],
            security_groups=security_groups)
        self.servers.append(server)
        return server

    def _get_server_key(self, server):
        return self.keypairs[server['key_name']]['private_key']

    def _check_public_network_connectivity(
            self, should_connect=True, msg=None,
            should_check_floating_ip_status=True, mtu=None):
        """Verifies connectivty to a VM via public network and floating IP

        and verifies floating IP has resource status is correct.

        :param should_connect: bool. determines if connectivity check is
        negative or positive.
        :param msg: Failure message to add to Error message. Should describe
        the place in the test scenario where the method was called,
        to indicate the context of the failure
        :param should_check_floating_ip_status: bool. should status of
        floating_ip be checked or not
        :param mtu: int. MTU network to use for connectivity validation
        """
        ssh_login = CONF.validation.image_ssh_user
        floating_ip, server = self.floating_ip_tuple
        ip_address = floating_ip['floating_ip_address']
        private_key = None
        floatingip_status = 'DOWN'
        if should_connect:
            private_key = self._get_server_key(server)
            floatingip_status = 'ACTIVE'

        # Check FloatingIP Status before initiating a connection
        if should_check_floating_ip_status:
            self.check_floating_ip_status(floating_ip, floatingip_status)

        message = 'Public network connectivity check failed'
        if msg:
            message += '. Reason: %s' % msg

        self.check_vm_connectivity(
            ip_address, ssh_login, private_key, should_connect,
            message, server, mtu=mtu)

    def _disassociate_floating_ips(self):
        floating_ip, _ = self.floating_ip_tuple
        floating_ip = self.floating_ips_client.update_floatingip(
            floating_ip['id'], port_id=None)['floatingip']
        self.assertIsNone(floating_ip['port_id'])
        self.floating_ip_tuple = Floating_IP_tuple(floating_ip, None)

    def _reassociate_floating_ips(self):
        floating_ip, server = self.floating_ip_tuple
        # create a new server for the floating ip
        server = self._create_server(self.network)
        port_id, _ = self.get_server_port_id_and_ip4(server)
        floating_ip = self.floating_ips_client.update_floatingip(
            floating_ip['id'], port_id=port_id)['floatingip']
        self.assertEqual(port_id, floating_ip['port_id'])
        self.floating_ip_tuple = Floating_IP_tuple(floating_ip, server)

    def _create_new_network(self, create_gateway=False):
        self.new_net = self.create_network()
        if create_gateway:
            self.new_subnet = self.create_subnet(
                network=self.new_net)
        else:
            self.new_subnet = self.create_subnet(
                network=self.new_net,
                gateway_ip=None)

    def _hotplug_server(self):
        old_floating_ip, server = self.floating_ip_tuple
        ip_address = old_floating_ip['floating_ip_address']
        private_key = self._get_server_key(server)
        ssh_client = self.get_remote_client(
            ip_address, private_key=private_key, server=server)
        old_nic_list = self._get_server_nics(ssh_client)
        # get a port from a list of one item
        port_list = self.os_admin.ports_client.list_ports(
            device_id=server['id'])['ports']
        self.assertEqual(1, len(port_list))
        old_port = port_list[0]
        interface = self.interface_client.create_interface(
            server_id=server['id'],
            net_id=self.new_net['id'])['interfaceAttachment']
        self.addCleanup(self.ports_client.wait_for_resource_deletion,
                        interface['port_id'])
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.interface_client.delete_interface,
                        server['id'], interface['port_id'])

        def check_ports():
            self.new_port_list = [
                port for port in
                self.os_admin.ports_client.list_ports(
                    device_id=server['id'])['ports']
                if port['id'] != old_port['id']
            ]
            return len(self.new_port_list) == 1

        if not test_utils.call_until_true(
                check_ports, CONF.network.build_timeout,
                CONF.network.build_interval):
            raise exceptions.TimeoutException(
                "No new port attached to the server in time (%s sec)! "
                "Old port: %s. Number of new ports: %d" % (
                    CONF.network.build_timeout, old_port,
                    len(self.new_port_list)))
        new_port = self.new_port_list[0]

        def check_new_nic():
            new_nic_list = self._get_server_nics(ssh_client)
            self.diff_list = [n for n in new_nic_list if n not in old_nic_list]
            return len(self.diff_list) == 1

        if not test_utils.call_until_true(
                check_new_nic, CONF.network.build_timeout,
                CONF.network.build_interval):
            raise exceptions.TimeoutException("Interface not visible on the "
                                              "guest after %s sec"
                                              % CONF.network.build_timeout)

        _, new_nic = self.diff_list[0]
        ip_output = ssh_client.exec_command('ip a')
        ip_address = new_port['fixed_ips'][0]['ip_address']
        ip_mask = CONF.network.project_network_mask_bits
        # check if the address is not already in use, if not, set it
        if ' ' + ip_address + '/' + str(ip_mask) not in ip_output:
            try:
                ssh_client.exec_command("sudo ip addr add %s/%s dev %s" % (
                                        ip_address, ip_mask, new_nic))
                ssh_client.exec_command("sudo ip link set %s up" % new_nic)
            except exceptions.SSHExecCommandFailed as exc:
                if 'RTNETLINK answers: File exists' in str(exc):
                    LOG.debug(
                        'IP address %(ip_address)s is already set in device '
                        '%(device)s\nPrevious "ip a" output: %(ip_output)s',
                        {'ip_address': ip_address, 'device': new_nic,
                         'ip_output': ip_output})
                else:
                    raise exc

    def _get_server_nics(self, ssh_client):
        reg = re.compile(r'(?P<num>\d+): (?P<nic_name>\w+)[@]?.*:')
        ipatxt = ssh_client.exec_command("ip address")
        return reg.findall(ipatxt)

    def _check_network_internal_connectivity(self, network,
                                             should_connect=True):
        """via ssh check VM internal connectivity:

        - ping internal gateway and DHCP port, implying in-tenant connectivity
        pinging both, because L3 and DHCP agents might be on different nodes
        - ping internal compute port, implying connectivity to other VMs on
        this network
        """
        floating_ip, server = self.floating_ip_tuple
        # get internal ports' ips:
        # get all network and compute ports in the new network
        internal_ips = (
            p['fixed_ips'][0]['ip_address'] for p in
            self.os_admin.ports_client.list_ports(
                project_id=server['tenant_id'],
                network_id=network['id'])['ports']
            if p['device_owner'].startswith('network') or
            p['device_owner'].startswith('compute')
        )

        self._check_server_connectivity(floating_ip,
                                        internal_ips,
                                        should_connect)

    def _check_network_external_connectivity(self):
        """ping default gateway to imply external connectivity"""
        if not CONF.network.public_network_id:
            msg = 'public network not defined.'
            LOG.info(msg)
            return

        # We ping the external IP from the instance using its floating IP
        # which is always IPv4, so we must only test connectivity to
        # external IPv4 IPs if the external network is dualstack.
        v4_subnets = [
            s for s in self.os_admin.subnets_client.list_subnets(
                network_id=CONF.network.public_network_id)['subnets']
            if s['ip_version'] == 4
        ]

        if len(v4_subnets) > 1:
            self.assertTrue(
                CONF.network.subnet_id,
                "Found %d subnets. Specify subnet using configuration "
                "option [network].subnet_id."
                % len(v4_subnets))
            subnet = self.os_admin.subnets_client.show_subnet(
                CONF.network.subnet_id)['subnet']
            external_ips = [subnet['gateway_ip']]
        else:
            external_ips = [v4_subnets[0]['gateway_ip']]

        self._check_server_connectivity(self.floating_ip_tuple.floating_ip,
                                        external_ips)

    def _check_server_connectivity(self, floating_ip, address_list,
                                   should_connect=True):
        ip_address = floating_ip['floating_ip_address']
        private_key = self._get_server_key(self.floating_ip_tuple.server)
        ssh_source = self.get_remote_client(
            ip_address, private_key=private_key,
            server=self.floating_ip_tuple.server)

        for remote_ip in address_list:
            self.check_remote_connectivity(ssh_source, remote_ip,
                                           should_connect)

    def _update_router_admin_state(self, router, admin_state_up):
        kwargs = dict(admin_state_up=admin_state_up)
        router = self.routers_client.update_router(
            router['id'], **kwargs)['router']
        self.assertEqual(admin_state_up, router['admin_state_up'])

    @decorators.attr(type='smoke')
    @decorators.idempotent_id('f323b3ba-82f8-4db7-8ea6-6a895869ec49')
    @utils.services('compute', 'network')
    def test_network_basic_ops(self):
        """Basic network operation test

        For a freshly-booted VM with an IP address ("port") on a given network:

        - the Tempest host can ping the IP address.  This implies, but
            does not guarantee (see the ssh check that follows), that the
            VM has been assigned the correct IP address and has
            connectivity to the Tempest host.

        - the Tempest host can perform key-based authentication to an
            ssh server hosted at the IP address.  This check guarantees
            that the IP address is associated with the target VM.

        - the Tempest host can ssh into the VM via the IP address and
            successfully execute the following:

            - ping an external IP address, implying external connectivity.

            - ping an external hostname, implying that dns is correctly
                configured.

            - ping an internal IP address, implying connectivity to another
               VM on the same network.

        - detach the floating-ip from the VM and verify that it becomes
            unreachable

        - associate detached floating ip to a new VM and verify connectivity.
            VMs are created with unique keypair so connectivity also asserts
            that floating IP is associated with the new VM instead of the old
            one

        Verifies that floating IP status is updated correctly after each change
        """
        self._setup_network_and_servers()
        self._check_public_network_connectivity(should_connect=True)
        self._check_network_internal_connectivity(network=self.network)
        self._check_network_external_connectivity()
        self._disassociate_floating_ips()
        self._check_public_network_connectivity(should_connect=False,
                                                msg="after disassociate "
                                                    "floating ip")
        self._reassociate_floating_ips()
        self._check_public_network_connectivity(should_connect=True,
                                                msg="after re-associate "
                                                    "floating ip")

    @decorators.idempotent_id('b158ea55-472e-4086-8fa9-c64ac0c6c1d0')
    @testtools.skipUnless(utils.is_extension_enabled('net-mtu', 'network'),
                          'No way to calculate MTU for networks')
    @decorators.attr(type='slow')
    @utils.services('compute', 'network')
    def test_mtu_sized_frames(self):
        """Validate that network MTU sized frames fit through."""
        self._setup_network_and_servers()
        # first check that connectivity works in general for the instance
        self._check_public_network_connectivity(should_connect=True)
        # now that we checked general connectivity, test that full size frames
        # can also pass between nodes
        self._check_public_network_connectivity(
            should_connect=True, mtu=self.network['mtu'])

    @decorators.idempotent_id('1546850e-fbaa-42f5-8b5f-03d8a6a95f15')
    @testtools.skipIf(CONF.network.shared_physical_network,
                      'Connectivity can only be tested when in a '
                      'multitenant network environment')
    @decorators.attr(type='slow')
    @utils.services('compute', 'network')
    def test_connectivity_between_vms_on_different_networks(self):
        """Test connectivity between VMs on different networks

        For a freshly-booted VM with an IP address ("port") on a given network:

        - the Tempest host can ping the IP address.
        - the Tempest host can ssh into the VM via the IP address and
            successfully execute the following:

            - ping an external IP address, implying external connectivity.
            - ping an external hostname, implying that dns is correctly
               configured.
            - ping an internal IP address, implying connectivity to another
               VM on the same network.

        - Create another network on the same tenant with subnet, create
            an VM on the new network.

            - Ping the new VM from previous VM failed since the new network
                was not attached to router yet.
            - Attach the new network to the router, Ping the new VM from
                previous VM succeed.

        """
        self._setup_network_and_servers()
        self._check_public_network_connectivity(should_connect=True)
        self._check_network_internal_connectivity(network=self.network)
        self._check_network_external_connectivity()
        self._create_new_network(create_gateway=True)
        new_server = self._create_server(self.new_net)
        new_server_ips = [addr['addr'] for addr in
                          new_server['addresses'][self.new_net['name']]]

        # Assert that pinging the new VM fails since the new network is not
        # connected to a router
        self._check_server_connectivity(self.floating_ip_tuple.floating_ip,
                                        new_server_ips, should_connect=False)
        router_id = self.router['id']
        self.routers_client.add_router_interface(
            router_id, subnet_id=self.new_subnet['id'])

        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.routers_client.remove_router_interface,
                        router_id, subnet_id=self.new_subnet['id'])

        self._check_server_connectivity(self.floating_ip_tuple.floating_ip,
                                        new_server_ips, should_connect=True)

    @decorators.idempotent_id('c5adff73-e961-41f1-b4a9-343614f18cfa')
    @testtools.skipUnless(CONF.compute_feature_enabled.interface_attach,
                          'NIC hotplug not available')
    @testtools.skipIf(CONF.network.port_vnic_type in ['direct', 'macvtap'],
                      'NIC hotplug not supported for '
                      'vnic_type direct or macvtap')
    @utils.services('compute', 'network')
    def test_hotplug_nic(self):
        """Test hotplug network interface

        1. Create a network and a VM.
        2. Check connectivity to the VM via a public network.
        3. Create a new network, with no gateway.
        4. Bring up a new interface
        5. check the VM reach the new network

        """
        self._setup_network_and_servers()
        self._check_public_network_connectivity(should_connect=True)
        self._create_new_network()
        self._hotplug_server()
        self._check_network_internal_connectivity(network=self.new_net)

    @decorators.idempotent_id('04b9fe4e-85e8-4aea-b937-ea93885ac59f')
    @testtools.skipIf(CONF.network.shared_physical_network,
                      'Router state can be altered only with multitenant '
                      'networks capabilities')
    @decorators.attr(type='slow')
    @utils.services('compute', 'network')
    def test_update_router_admin_state(self):
        """Test to update admin state up of router

        1. Check public connectivity before updating
                admin_state_up attribute of router to False
        2. Check public connectivity after updating
                admin_state_up attribute of router to False
        3. Check public connectivity after updating
                admin_state_up attribute of router to True
        """
        self._setup_network_and_servers()
        self._check_public_network_connectivity(
            should_connect=True, msg="before updating "
            "admin_state_up of router to False")
        self._update_router_admin_state(self.router, False)
        # TODO(alokmaurya): Remove should_check_floating_ip_status=False check
        # once bug 1396310 is fixed

        self._check_public_network_connectivity(
            should_connect=False, msg="after updating "
            "admin_state_up of router to False",
            should_check_floating_ip_status=False)
        self._update_router_admin_state(self.router, True)
        self._check_public_network_connectivity(
            should_connect=True, msg="after updating "
            "admin_state_up of router to True")

    @decorators.idempotent_id('d8bb918e-e2df-48b2-97cd-b73c95450980')
    @testtools.skipIf(CONF.network.shared_physical_network,
                      'network isolation not available')
    @testtools.skipUnless(CONF.scenario.dhcp_client,
                          "DHCP client is not available.")
    @decorators.attr(type='slow')
    @utils.services('compute', 'network')
    def test_subnet_details(self):
        """Tests that subnet's extra configuration details are affecting VMs.

        This test relies on non-shared, isolated tenant networks.

        NOTE: Neutron subnets push data to servers via dhcp-agent, so any
        update in subnet requires server to actively renew its DHCP lease.

        1. Configure subnet with dns nameserver
        2. retrieve the VM's configured dns and verify it matches the one
           configured for the subnet.
        3. update subnet's dns
        4. retrieve the VM's configured dns and verify it matches the new one
           configured for the subnet.

        TODO(yfried): add host_routes

        any resolution check would be testing either:

        * l3 forwarding (tested in test_network_basic_ops)
        * Name resolution of an external DNS nameserver - out of scope for
          Tempest
        """
        # this test check only updates (no actual resolution) so using
        # arbitrary ip addresses as nameservers, instead of parsing CONF
        initial_dns_server = '1.2.3.4'
        alt_dns_server = '9.8.7.6'

        # Original timeouts are suggested by salvatore-orlando in
        # https://bugs.launchpad.net/neutron/+bug/1412325/comments/3
        #
        # Compared to that renew_delay was increased, because
        # busybox's udhcpc accepts SIGUSR1 as a renew request. Internally
        # it goes into RENEW_REQUESTED state. If it receives a 2nd SIGUSR1
        # signal while in that state then it calls the deconfig script
        # ("/sbin/cirros-dhcpc deconfig" in sufficiently new cirros versions)
        # which leads to the address being transiently deconfigured which
        # for our case is unwanted.
        renew_delay = 3 * CONF.network.build_interval
        renew_timeout = CONF.network.build_timeout

        self._setup_network_and_servers(dns_nameservers=[initial_dns_server])
        self._check_public_network_connectivity(should_connect=True)

        floating_ip, server = self.floating_ip_tuple
        ip_address = floating_ip['floating_ip_address']
        private_key = self._get_server_key(server)
        ssh_client = self.get_remote_client(
            ip_address, private_key=private_key, server=server)

        dns_servers = [initial_dns_server]
        servers = ssh_client.get_dns_servers()
        self.assertEqual(set(dns_servers), set(servers),
                         'Looking for servers: {trgt_serv}. '
                         'Retrieved DNS nameservers: {act_serv} '
                         'From host: {host}.'
                         .format(host=ssh_client.ssh_client.host,
                                 act_serv=servers,
                                 trgt_serv=dns_servers))

        self.subnet = self.subnets_client.update_subnet(
            self.subnet['id'], dns_nameservers=[alt_dns_server])['subnet']

        # asserts that Neutron DB has updated the nameservers
        self.assertEqual([alt_dns_server], self.subnet['dns_nameservers'],
                         "Failed to update subnet's nameservers")

        def check_new_dns_server():
            # NOTE: Server needs to renew its dhcp lease in order to get new
            # definitions from subnet
            # NOTE(amuller): we are renewing the lease as part of the retry
            # because Neutron updates dnsmasq asynchronously after the
            # subnet-update API call returns.
            ssh_client.renew_lease(fixed_ip=floating_ip['fixed_ip_address'],
                                   dhcp_client=CONF.scenario.dhcp_client)
            if ssh_client.get_dns_servers() != [alt_dns_server]:
                LOG.debug("Failed to update DNS nameservers")
                return False
            return True

        self.assertTrue(test_utils.call_until_true(check_new_dns_server,
                                                   renew_timeout,
                                                   renew_delay),
                        msg="DHCP renewal failed to fetch "
                            "new DNS nameservers")

    @decorators.idempotent_id('f5dfcc22-45fd-409f-954c-5bd500d7890b')
    @testtools.skipUnless(CONF.network_feature_enabled.port_admin_state_change,
                          "Changing a port's admin state is not supported "
                          "by the test environment")
    @decorators.attr(type='slow')
    @utils.services('compute', 'network')
    def test_update_instance_port_admin_state(self):
        """Test to update admin_state_up attribute of instance port

        1. Check public and project connectivity before updating
                admin_state_up attribute of instance port to False
        2. Check public and project connectivity after updating
                admin_state_up attribute of instance port to False
        3. Check public and project connectivity after updating
                admin_state_up attribute of instance port to True
        """
        self._setup_network_and_servers()
        _, server = self.floating_ip_tuple
        server_id = server['id']
        port_id = self.os_admin.ports_client.list_ports(
            device_id=server_id)['ports'][0]['id']
        server_pip = server['addresses'][self.network['name']][0]['addr']

        server2 = self._create_server(self.network)
        server2_fip = self.create_floating_ip(server2)

        private_key = self._get_server_key(server2)
        ssh_client = self.get_remote_client(server2_fip['floating_ip_address'],
                                            private_key=private_key,
                                            server=server2)

        self._check_public_network_connectivity(
            should_connect=True, msg="before updating "
            "admin_state_up of instance port to False")
        self.check_remote_connectivity(ssh_client, dest=server_pip,
                                       should_succeed=True)
        self.ports_client.update_port(port_id, admin_state_up=False)
        self._check_public_network_connectivity(
            should_connect=False, msg="after updating "
            "admin_state_up of instance port to False",
            should_check_floating_ip_status=False)
        self.check_remote_connectivity(ssh_client, dest=server_pip,
                                       should_succeed=False)
        self.ports_client.update_port(port_id, admin_state_up=True)
        self._check_public_network_connectivity(
            should_connect=True, msg="after updating "
            "admin_state_up of instance port to True")
        self.check_remote_connectivity(ssh_client, dest=server_pip,
                                       should_succeed=True)

    @decorators.idempotent_id('759462e1-8535-46b0-ab3a-33aa45c55aaa')
    @decorators.attr(type='slow')
    @utils.services('compute', 'network')
    def test_preserve_preexisting_port(self):
        """Test preserve pre-existing port

        Tests that a pre-existing port provided on server boot is not deleted
        if the server is deleted.

        Nova should unbind the port from the instance on delete if the port was
        not created by Nova as part of the boot request.

        We should also be able to boot another server with the same port.
        """
        # Setup the network, create a port and boot the server from that port.
        self._setup_network_and_servers(boot_with_port=True)
        _, server = self.floating_ip_tuple
        self.assertEqual(1, len(self.ports),
                         'There should only be one port created for '
                         'server %s.' % server['id'])
        port_id = self.ports[0]['port']
        self.assertIsNotNone(port_id,
                             'Server should have been created from a '
                             'pre-existing port.')
        # Assert the port is bound to the server.
        port_list = self.os_admin.ports_client.list_ports(
            device_id=server['id'], network_id=self.network['id'])['ports']
        self.assertEqual(1, len(port_list),
                         'There should only be one port created for '
                         'server %s.' % server['id'])
        self.assertEqual(port_id, port_list[0]['id'])
        # Delete the server.
        self.servers_client.delete_server(server['id'])
        waiters.wait_for_server_termination(self.servers_client, server['id'])
        # Assert the port still exists on the network but is unbound from
        # the deleted server.
        port = self.ports_client.show_port(port_id)['port']
        self.assertEqual(self.network['id'], port['network_id'])
        self.assertEqual('', port['device_id'])
        self.assertEqual('', port['device_owner'])

        # Boot another server with the same port to make sure nothing was
        # left around that could cause issues.
        server = self._create_server(self.network, port['id'])
        port_list = self.os_admin.ports_client.list_ports(
            device_id=server['id'], network_id=self.network['id'])['ports']
        self.assertEqual(1, len(port_list),
                         'There should only be one port created for '
                         'server %s.' % server['id'])
        self.assertEqual(port['id'], port_list[0]['id'])

    @utils.requires_ext(service='network', extension='l3_agent_scheduler')
    @decorators.idempotent_id('2e788c46-fb3f-4ac9-8f82-0561555bea73')
    @decorators.attr(type='slow')
    @utils.services('compute', 'network')
    def test_router_rescheduling(self):
        """Tests that router can be removed from agent and add to a new agent.

        1. Verify connectivity
        2. Remove router from all l3-agents
        3. Verify connectivity is down
        4. Assign router to new l3-agent (or old one if no new agent is
           available)
        5. Verify connectivity
        """

        # TODO(yfried): refactor this test to be used for other agents (dhcp)
        # as well

        list_hosts = (self.os_admin.routers_client.
                      list_l3_agents_hosting_router)
        schedule_router = (self.os_admin.network_agents_client.
                           create_router_on_l3_agent)
        unschedule_router = (self.os_admin.network_agents_client.
                             delete_router_from_l3_agent)

        agent_list_alive = set(
            a["id"] for a in
            self.os_admin.network_agents_client.list_agents(
                agent_type="L3 agent")['agents'] if a["alive"] is True
        )
        self._setup_network_and_servers()

        # NOTE(kevinbenton): we have to use the admin credentials to check
        # for the distributed flag because self.router only has a project view.
        admin = self.os_admin.routers_client.show_router(
            self.router['id'])
        if admin['router'].get('distributed', False):
            msg = "Rescheduling test does not apply to distributed routers."
            raise self.skipException(msg)

        self._check_public_network_connectivity(should_connect=True)

        # remove resource from agents
        hosting_agents = set(a["id"] for a in
                             list_hosts(self.router['id'])['agents'])
        no_migration = agent_list_alive == hosting_agents
        LOG.info("Router will be assigned to {mig} hosting agent".
                 format(mig="the same" if no_migration else "a new"))

        for hosting_agent in hosting_agents:
            unschedule_router(hosting_agent, self.router['id'])
            self.assertNotIn(hosting_agent,
                             [a["id"] for a in
                              list_hosts(self.router['id'])['agents']],
                             'unscheduling router failed')

        # verify resource is un-functional
        self._check_public_network_connectivity(
            should_connect=False,
            msg='after router unscheduling',
        )

        # schedule resource to new agent
        target_agent = list(hosting_agents if no_migration else
                            agent_list_alive - hosting_agents)[0]
        schedule_router(target_agent,
                        router_id=self.router['id'])
        self.assertEqual(
            target_agent,
            list_hosts(self.router['id'])['agents'][0]['id'],
            "Router failed to reschedule. Hosting agent doesn't match "
            "target agent")

        # verify resource is functional
        self._check_public_network_connectivity(
            should_connect=True,
            msg='After router rescheduling')

    @utils.requires_ext(service='network', extension='port-security')
    @testtools.skipUnless(CONF.compute_feature_enabled.interface_attach,
                          'NIC hotplug not available')
    @decorators.idempotent_id('7c0bb1a2-d053-49a4-98f9-ca1a1d849f63')
    @decorators.attr(type='slow')
    @utils.services('compute', 'network')
    def test_port_security_macspoofing_port(self):
        """Tests port_security extension enforces mac spoofing

        Neutron security groups always apply anti-spoof rules on the VMs. This
        allows traffic to originate and terminate at the VM as expected, but
        prevents traffic to pass through the VM. Anti-spoof rules are not
        required in cases where the VM routes traffic through it.

        The test steps are:

        1. Create a new network.
        2. Connect (hotplug) the VM to a new network.
        3. Check the VM can ping a server on the new network ("peer")
        4. Spoof the mac address of the new VM interface.
        5. Check the Security Group enforces mac spoofing and blocks pings via
           spoofed interface (VM cannot ping the peer).
        6. Disable port-security of the spoofed port- set the flag to false.
        7. Retest 3rd step and check that the Security Group allows pings via
           the spoofed interface.
        """

        spoof_mac = "00:00:00:00:00:01"

        # Create server
        self._setup_network_and_servers()
        self._check_public_network_connectivity(should_connect=True)
        self._create_new_network()
        self._hotplug_server()
        fip, server = self.floating_ip_tuple
        new_ports = self.os_admin.ports_client.list_ports(
            device_id=server["id"], network_id=self.new_net["id"])['ports']
        spoof_port = new_ports[0]
        private_key = self._get_server_key(server)
        ssh_client = self.get_remote_client(fip['floating_ip_address'],
                                            private_key=private_key,
                                            server=server)
        spoof_nic = ssh_client.get_nic_name_by_mac(spoof_port["mac_address"])
        peer = self._create_server(self.new_net)
        peer_address = peer['addresses'][self.new_net['name']][0]['addr']
        self.check_remote_connectivity(ssh_client, dest=peer_address,
                                       nic=spoof_nic, should_succeed=True)
        # Set a mac address by making nic down temporary
        cmd = ("sudo ip link set {nic} down;"
               "sudo ip link set dev {nic} address {mac};"
               "sudo ip link set {nic} up").format(nic=spoof_nic,
                                                   mac=spoof_mac)
        ssh_client.exec_command(cmd)

        new_mac = ssh_client.get_mac_address(nic=spoof_nic)
        self.assertEqual(spoof_mac, new_mac)
        self.check_remote_connectivity(ssh_client, dest=peer_address,
                                       nic=spoof_nic, should_succeed=False)
        self.ports_client.update_port(spoof_port["id"],
                                      port_security_enabled=False,
                                      security_groups=[])
        self.check_remote_connectivity(ssh_client, dest=peer_address,
                                       nic=spoof_nic, should_succeed=True)



# Copyright 2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from tempest.lib.services.identity.v3 import domain_configuration_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestDomainConfigurationClient(base.BaseServiceTest):

    FAKE_CONFIG_SETTINGS = {
        "config": {
            "identity": {
                "driver": "ldap"
            },
            "ldap": {
                "url": "ldap://localhost",
                "user": "",
                "suffix": "cn=example,cn=com",
            }
        }
    }

    FAKE_DOMAIN_ID = '07ef7d04-2941-4bee-8551-f79f08a021de'

    def setUp(self):
        super(TestDomainConfigurationClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.client = domain_configuration_client.DomainConfigurationClient(
            fake_auth, 'identity', 'regionOne')

    def _test_show_default_config_settings(self, bytes_body=False):
        self.check_service_client_function(
            self.client.show_default_config_settings,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_CONFIG_SETTINGS,
            bytes_body)

    def _test_show_default_group_config(self, bytes_body=False):
        self.check_service_client_function(
            self.client.show_default_group_config,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_CONFIG_SETTINGS['config']['ldap'],
            bytes_body,
            group='ldap')

    def _test_show_default_group_option(self, bytes_body=False):
        self.check_service_client_function(
            self.client.show_default_group_option,
            'tempest.lib.common.rest_client.RestClient.get',
            {'driver': 'ldap'},
            bytes_body,
            group='identity',
            option='driver')

    def _test_show_domain_group_option_config(self, bytes_body=False):
        self.check_service_client_function(
            self.client.show_domain_group_option_config,
            'tempest.lib.common.rest_client.RestClient.get',
            {'driver': 'ldap'},
            bytes_body,
            domain_id=self.FAKE_DOMAIN_ID,
            group='identity',
            option='driver')

    def _test_update_domain_group_option_config(self, bytes_body=False):
        self.check_service_client_function(
            self.client.update_domain_group_option_config,
            'tempest.lib.common.rest_client.RestClient.patch',
            self.FAKE_CONFIG_SETTINGS,
            bytes_body,
            domain_id=self.FAKE_DOMAIN_ID,
            group='identity',
            option='driver',
            url='http://myldap/my_other_root')

    def _test_show_domain_group_config(self, bytes_body=False):
        self.check_service_client_function(
            self.client.show_domain_group_config,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_CONFIG_SETTINGS['config']['ldap'],
            bytes_body,
            domain_id=self.FAKE_DOMAIN_ID,
            group='ldap')

    def _test_update_domain_group_config(self, bytes_body=False):
        self.check_service_client_function(
            self.client.update_domain_group_config,
            'tempest.lib.common.rest_client.RestClient.patch',
            self.FAKE_CONFIG_SETTINGS['config']['ldap'],
            bytes_body,
            domain_id=self.FAKE_DOMAIN_ID,
            group='ldap',
            **self.FAKE_CONFIG_SETTINGS['config'])

    def _test_create_domain_config(self, bytes_body=False):
        self.check_service_client_function(
            self.client.create_domain_config,
            'tempest.lib.common.rest_client.RestClient.put',
            self.FAKE_CONFIG_SETTINGS,
            bytes_body,
            domain_id=self.FAKE_DOMAIN_ID,
            status=201)

    def _test_show_domain_config(self, bytes_body=False):
        self.check_service_client_function(
            self.client.show_domain_config,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_CONFIG_SETTINGS,
            bytes_body,
            domain_id=self.FAKE_DOMAIN_ID)

    def _test_update_domain_config(self, bytes_body=False):
        self.check_service_client_function(
            self.client.update_domain_config,
            'tempest.lib.common.rest_client.RestClient.patch',
            self.FAKE_CONFIG_SETTINGS,
            bytes_body,
            domain_id=self.FAKE_DOMAIN_ID)

    def test_show_default_config_settings_with_str_body(self):
        self._test_show_default_config_settings()

    def test_show_default_config_settings_with_bytes_body(self):
        self._test_show_default_config_settings(bytes_body=True)

    def test_show_default_group_config_with_str_body(self):
        self._test_show_default_group_config()

    def test_show_default_group_config_with_bytes_body(self):
        self._test_show_default_group_config(bytes_body=True)

    def test_show_default_group_option_with_str_body(self):
        self._test_show_default_group_option()

    def test_show_default_group_option_with_bytes_body(self):
        self._test_show_default_group_option(bytes_body=True)

    def test_show_domain_group_option_config_with_str_body(self):
        self._test_show_domain_group_option_config()

    def test_show_domain_group_option_config_with_bytes_body(self):
        self._test_show_domain_group_option_config(bytes_body=True)

    def test_update_domain_group_option_config_with_str_body(self):
        self._test_update_domain_group_option_config()

    def test_update_domain_group_option_config_with_bytes_body(self):
        self._test_update_domain_group_option_config(bytes_body=True)

    def test_delete_domain_group_option_config(self):
        self.check_service_client_function(
            self.client.delete_domain_group_option_config,
            'tempest.lib.common.rest_client.RestClient.delete',
            {},
            status=204,
            domain_id=self.FAKE_DOMAIN_ID,
            group='identity',
            option='driver')

    def test_show_domain_group_config_with_str_body(self):
        self._test_show_domain_group_config()

    def test_show_domain_group_config_with_bytes_body(self):
        self._test_show_domain_group_config(bytes_body=True)

    def test_test_update_domain_group_config_with_str_body(self):
        self._test_update_domain_group_config()

    def test_update_domain_group_config_with_bytes_body(self):
        self._test_update_domain_group_config(bytes_body=True)

    def test_delete_domain_group_config(self):
        self.check_service_client_function(
            self.client.delete_domain_group_config,
            'tempest.lib.common.rest_client.RestClient.delete',
            {},
            status=204,
            domain_id=self.FAKE_DOMAIN_ID,
            group='identity')

    def test_create_domain_config_with_str_body(self):
        self._test_create_domain_config()

    def test_create_domain_config_with_bytes_body(self):
        self._test_create_domain_config(bytes_body=True)

    def test_show_domain_config_with_str_body(self):
        self._test_show_domain_config()

    def test_show_domain_config_with_bytes_body(self):
        self._test_show_domain_config(bytes_body=True)

    def test_update_domain_config_with_str_body(self):
        self._test_update_domain_config()

    def test_update_domain_config_with_bytes_body(self):
        self._test_update_domain_config(bytes_body=True)

    def test_delete_domain_config(self):
        self.check_service_client_function(
            self.client.delete_domain_config,
            'tempest.lib.common.rest_client.RestClient.delete',
            {},
            status=204,
            domain_id=self.FAKE_DOMAIN_ID)



# Copyright (C) 2017 Dell Inc. or its subsidiaries.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from urllib import parse as urllib

from oslo_serialization import jsonutils as json

from tempest.lib.api_schema.response.volume import group_types as schema
from tempest.lib.common import rest_client
from tempest.lib.services.volume import base_client


class GroupTypesClient(base_client.BaseClient):
    """Client class to send CRUD Volume V3 Group Types API requests"""

    @property
    def resource_type(self):
        """Returns the primary type of resource this client works with."""
        return 'group-type'

    def create_group_type(self, **kwargs):
        """Create group_type.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/block-storage/v3/#create-group-type
        """
        post_body = json.dumps({'group_type': kwargs})
        resp, body = self.post('group_types', post_body)
        body = json.loads(body)
        self.validate_response(schema.create_group_type, resp, body)
        return rest_client.ResponseBody(resp, body)

    def delete_group_type(self, group_type_id):
        """Deletes the specified group_type."""
        resp, body = self.delete("group_types/%s" % group_type_id)
        self.validate_response(schema.delete_group_type, resp, body)
        return rest_client.ResponseBody(resp, body)

    def list_group_types(self, **params):
        """List all the group_types created.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/block-storage/v3/#list-group-types
        """
        url = 'group_types'
        if params:
            url += '?%s' % urllib.urlencode(params)

        resp, body = self.get(url)
        body = json.loads(body)
        self.validate_response(schema.list_group_types, resp, body)
        return rest_client.ResponseBody(resp, body)

    def show_default_group_type(self):
        """Returns the details of default group_type.

        For more information, please refer to the official API reference:
        https://docs.openstack.org/api-ref/block-storage/v3/#show-default-group-type-details
        """
        url = 'group_types/default'
        resp, body = self.get(url)
        body = json.loads(body)
        self.validate_response(schema.show_default_group_type, resp, body)
        return rest_client.ResponseBody(resp, body)

    def show_group_type(self, group_type_id):
        """Returns the details of a single group_type.

        For more information, please refer to the official API reference:
        https://docs.openstack.org/api-ref/block-storage/v3/#show-group-type-details
        """
        url = "group_types/%s" % group_type_id
        resp, body = self.get(url)
        body = json.loads(body)
        self.validate_response(schema.show_group_type, resp, body)
        return rest_client.ResponseBody(resp, body)

    def update_group_type(self, group_type_id, **kwargs):
        """Updates a group type.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/block-storage/v3/#update-group-type
        """
        post_body = json.dumps({'group_type': kwargs})
        resp, body = self.put('group_types/%s' % group_type_id, post_body)
        body = json.loads(body)
        self.validate_response(schema.update_group_type, resp, body)
        return rest_client.ResponseBody(resp, body)

    def create_or_update_group_type_specs(self, group_type_id, group_specs):
        """Creates new group specs or updates existing group specs.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/block-storage/v3/#create-or-update-group-specs-for-a-group-type
        """
        url = "group_types/%s/group_specs" % group_type_id
        post_body = json.dumps({'group_specs': group_specs})
        resp, body = self.post(url, post_body)
        body = json.loads(body)
        self.validate_response(
            schema.create_or_update_group_type_specs, resp, body)
        return rest_client.ResponseBody(resp, body)

    def list_group_type_specs(self, group_type_id):
        """Lists all group specs for a given group type."""
        url = 'group_types/%s/group_specs' % group_type_id
        resp, body = self.get(url)
        body = json.loads(body)
        self.validate_response(schema.list_group_type_specs, resp, body)
        return rest_client.ResponseBody(resp, body)

    def show_group_type_specs_item(self, group_type_id, spec_id):
        """Shows specified item of group specs for a given group type."""
        url = "group_types/%s/group_specs/%s" % (group_type_id, spec_id)
        resp, body = self.get(url)
        body = json.loads(body)
        self.validate_response(schema.show_group_type_specs_item, resp, body)
        return rest_client.ResponseBody(resp, body)

    def update_group_type_specs_item(self, group_type_id, spec_id, spec):
        """Updates specified item of group specs for a given group type.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/block-storage/v3/#update-one-specific-group-spec-for-a-group-type
        """
        url = "group_types/%s/group_specs/%s" % (group_type_id, spec_id)
        put_body = json.dumps(spec)
        resp, body = self.put(url, put_body)
        body = json.loads(body)
        self.validate_response(schema.update_group_type_specs_item, resp, body)
        return rest_client.ResponseBody(resp, body)

    def delete_group_type_specs_item(self, group_type_id, spec_id):
        """Deletes specified item of group specs for a given group type."""
        resp, body = self.delete("group_types/%s/group_specs/%s" % (
            group_type_id, spec_id))
        self.validate_response(schema.delete_group_type_specs_item, resp, body)
        return rest_client.ResponseBody(resp, body)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from urllib import parse as urllib

from oslo_serialization import jsonutils as json

from tempest.lib.common import rest_client
from tempest.lib import exceptions as lib_exc


class SnapshotsClient(rest_client.RestClient):
    """Client class to send CRUD Volume V1 API requests."""

    create_resp = 200

    def list_snapshots(self, detail=False, **params):
        """List all the snapshot.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/block-storage/v2/#list-snapshots
        https://docs.openstack.org/api-ref/block-storage/v2/#list-snapshots-with-details
        """
        url = 'snapshots'
        if detail:
            url += '/detail'
        if params:
            url += '?%s' % urllib.urlencode(params)

        resp, body = self.get(url)
        body = json.loads(body)
        self.expected_success(200, resp.status)
        return rest_client.ResponseBody(resp, body)

    def show_snapshot(self, snapshot_id):
        """Returns the details of a single snapshot.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/block-storage/v2/#show-snapshot-details
        """
        url = "snapshots/%s" % snapshot_id
        resp, body = self.get(url)
        body = json.loads(body)
        self.expected_success(200, resp.status)
        return rest_client.ResponseBody(resp, body)

    def create_snapshot(self, **kwargs):
        """Creates a new snapshot.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/block-storage/v2/#create-snapshot
        """
        post_body = json.dumps({'snapshot': kwargs})
        resp, body = self.post('snapshots', post_body)
        body = json.loads(body)
        self.expected_success(self.create_resp, resp.status)
        return rest_client.ResponseBody(resp, body)

    def delete_snapshot(self, snapshot_id):
        """Delete Snapshot.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/block-storage/v2/#delete-snapshot
        """
        resp, body = self.delete("snapshots/%s" % snapshot_id)
        self.expected_success(202, resp.status)
        return rest_client.ResponseBody(resp, body)

    def is_resource_deleted(self, id):
        try:
            self.show_snapshot(id)
        except lib_exc.NotFound:
            return True
        return False

    @property
    def resource_type(self):
        """Returns the primary type of resource this client works with."""
        return 'volume-snapshot'

    def reset_snapshot_status(self, snapshot_id, status):
        """Reset the specified snapshot's status."""
        post_body = json.dumps({'os-reset_status': {"status": status}})
        resp, body = self.post('snapshots/%s/action' % snapshot_id, post_body)
        self.expected_success(202, resp.status)
        return rest_client.ResponseBody(resp, body)

    def update_snapshot_status(self, snapshot_id, **kwargs):
        """Update the specified snapshot's status."""
        # TODO(gmann): api-site doesn't contain doc ref
        # for this API. After fixing the api-site, we need to
        # add the link here.
        # Bug https://bugs.launchpad.net/openstack-api-site/+bug/1532645

        post_body = json.dumps({'os-update_snapshot_status': kwargs})
        url = 'snapshots/%s/action' % snapshot_id
        resp, body = self.post(url, post_body)
        self.expected_success(202, resp.status)
        return rest_client.ResponseBody(resp, body)

    def create_snapshot_metadata(self, snapshot_id, metadata):
        """Create metadata for the snapshot."""
        put_body = json.dumps({'metadata': metadata})
        url = "snapshots/%s/metadata" % snapshot_id
        resp, body = self.post(url, put_body)
        body = json.loads(body)
        self.expected_success(200, resp.status)
        return rest_client.ResponseBody(resp, body)

    def update_snapshot(self, snapshot_id, **kwargs):
        """Updates a snapshot.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/block-storage/v2/#update-snapshot
        """
        put_body = json.dumps({'snapshot': kwargs})
        resp, body = self.put('snapshots/%s' % snapshot_id, put_body)
        body = json.loads(body)
        self.expected_success(200, resp.status)
        return rest_client.ResponseBody(resp, body)

    def show_snapshot_metadata(self, snapshot_id):
        """Get metadata of the snapshot.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/block-storage/v2/#show-snapshot-metadata
        """
        url = "snapshots/%s/metadata" % snapshot_id
        resp, body = self.get(url)
        body = json.loads(body)
        self.expected_success(200, resp.status)
        return rest_client.ResponseBody(resp, body)

    def update_snapshot_metadata(self, snapshot_id, **kwargs):
        """Update metadata for the snapshot.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/block-storage/v2/#update-snapshot-metadata
        """
        put_body = json.dumps(kwargs)
        url = "snapshots/%s/metadata" % snapshot_id
        resp, body = self.put(url, put_body)
        body = json.loads(body)
        self.expected_success(200, resp.status)
        return rest_client.ResponseBody(resp, body)

    def update_snapshot_metadata_item(self, snapshot_id, id, **kwargs):
        """Update metadata item for the snapshot."""
        # TODO(piyush): Current api-site doesn't contain this API description.
        # After fixing the api-site, we need to fix here also for putting the
        # link to api-site.
        # LP: https://bugs.launchpad.net/openstack-api-site/+bug/1529064
        put_body = json.dumps(kwargs)
        url = "snapshots/%s/metadata/%s" % (snapshot_id, id)
        resp, body = self.put(url, put_body)
        body = json.loads(body)
        self.expected_success(200, resp.status)
        return rest_client.ResponseBody(resp, body)

    def delete_snapshot_metadata_item(self, snapshot_id, id):
        """Delete metadata item for the snapshot."""
        url = "snapshots/%s/metadata/%s" % (snapshot_id, id)
        resp, body = self.delete(url)
        self.expected_success(200, resp.status)
        return rest_client.ResponseBody(resp, body)

    def force_delete_snapshot(self, snapshot_id):
        """Force Delete Snapshot."""
        post_body = json.dumps({'os-force_delete': {}})
        resp, body = self.post('snapshots/%s/action' % snapshot_id, post_body)
        self.expected_success(202, resp.status)
        return rest_client.ResponseBody(resp, body)



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import time

import io
import testtools

from tempest.api.compute import base
from tempest.common import image as common_image
from tempest.common import waiters
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators
from tempest.lib import exceptions

CONF = config.CONF


class ListImageFiltersTestJSON(base.BaseV2ComputeTest):
    """Test listing server images with compute microversion less than 2.36"""

    max_microversion = '2.35'

    @classmethod
    def skip_checks(cls):
        super(ListImageFiltersTestJSON, cls).skip_checks()
        if not CONF.service_available.glance:
            skip_msg = ("%s skipped as glance is not available" % cls.__name__)
            raise cls.skipException(skip_msg)

    @classmethod
    def setup_clients(cls):
        super(ListImageFiltersTestJSON, cls).setup_clients()
        cls.client = cls.compute_images_client
        # Check if glance v1 is available to determine which client to use. We
        # prefer glance v1 for the compute API tests since the compute image
        # API proxy was written for glance v1.
        if CONF.image_feature_enabled.api_v1:
            cls.glance_client = cls.os_primary.image_client
        elif CONF.image_feature_enabled.api_v2:
            cls.glance_client = cls.os_primary.image_client_v2
        else:
            raise exceptions.InvalidConfiguration(
                'Either api_v1 or api_v2 must be True in '
                '[image-feature-enabled].')

    @classmethod
    def resource_setup(cls):
        super(ListImageFiltersTestJSON, cls).resource_setup()

        def _create_image():
            params = {
                'name': data_utils.rand_name(cls.__name__ + '-image'),
                'container_format': 'bare',
                'disk_format': 'raw'
            }
            if CONF.image_feature_enabled.api_v1:
                params.update({'is_public': False})
                params = {'headers':
                          common_image.image_meta_to_headers(**params)}
            else:
                params.update({'visibility': 'private'})

            body = cls.glance_client.create_image(**params)
            body = body['image'] if 'image' in body else body
            image_id = body['id']
            cls.addClassResourceCleanup(
                test_utils.call_and_ignore_notfound_exc,
                cls.compute_images_client.delete_image,
                image_id)
            # Wait 1 second between creation and upload to ensure a delta
            # between created_at and updated_at.
            time.sleep(1)
            image_file = io.BytesIO((b'*' * 1024))
            if CONF.image_feature_enabled.api_v1:
                cls.glance_client.update_image(image_id, data=image_file)
            else:
                cls.glance_client.store_image_file(image_id, data=image_file)
            waiters.wait_for_image_status(cls.client, image_id, 'ACTIVE')
            body = cls.client.show_image(image_id)['image']
            return body

        # Create non-snapshot images via glance
        cls.image1 = _create_image()
        cls.image1_id = cls.image1['id']
        cls.image2 = _create_image()
        cls.image2_id = cls.image2['id']
        cls.image3 = _create_image()
        cls.image3_id = cls.image3['id']

        if not CONF.compute_feature_enabled.snapshot:
            return

        # Create instances and snapshots via nova
        cls.server1 = cls.create_test_server()
        cls.server2 = cls.create_test_server(wait_until='ACTIVE')
        # NOTE(sdague) this is faster than doing the sync wait_util on both
        waiters.wait_for_server_status(cls.servers_client,
                                       cls.server1['id'], 'ACTIVE')

        # Create images to be used in the filter tests
        cls.snapshot1 = cls.create_image_from_server(
            cls.server1['id'], wait_until='ACTIVE')
        cls.snapshot1_id = cls.snapshot1['id']

        # Servers have a hidden property for when they are being imaged
        # Performing back-to-back create image calls on a single
        # server will sometimes cause failures
        cls.snapshot3 = cls.create_image_from_server(
            cls.server2['id'], wait_until='ACTIVE')
        cls.snapshot3_id = cls.snapshot3['id']

        # Wait for the server to be active after the image upload
        cls.snapshot2 = cls.create_image_from_server(
            cls.server1['id'], wait_until='ACTIVE')
        cls.snapshot2_id = cls.snapshot2['id']

    @decorators.idempotent_id('a3f5b513-aeb3-42a9-b18e-f091ef73254d')
    def test_list_images_filter_by_status(self):
        """Test listing server images filtered by image status

        The list of images should contain only images with the
        provided image status.
        """
        params = {'status': 'ACTIVE'}
        images = self.client.list_images(**params)['images']

        self.assertNotEmpty([i for i in images if i['id'] == self.image1_id])
        self.assertNotEmpty([i for i in images if i['id'] == self.image2_id])
        self.assertNotEmpty([i for i in images if i['id'] == self.image3_id])

    @decorators.idempotent_id('33163b73-79f5-4d07-a7ea-9213bcc468ff')
    def test_list_images_filter_by_name(self):
        """Test listing server images filtered by image name

        The list of images should contain only images with the
        provided image name.
        """
        params = {'name': self.image1['name']}
        images = self.client.list_images(**params)['images']

        self.assertNotEmpty([i for i in images if i['id'] == self.image1_id])
        self.assertEmpty([i for i in images if i['id'] == self.image2_id])
        self.assertEmpty([i for i in images if i['id'] == self.image3_id])

    @decorators.idempotent_id('9f238683-c763-45aa-b848-232ec3ce3105')
    @testtools.skipUnless(CONF.compute_feature_enabled.snapshot,
                          'Snapshotting is not available.')
    def test_list_images_filter_by_server_id(self):
        """Test listing images filtered by server id

        The list of images should contain only images with the
        provided server id.
        """
        params = {'server': self.server1['id']}
        images = self.client.list_images(**params)['images']

        self.assertNotEmpty([i for i in images
                             if i['id'] == self.snapshot1_id],
                            "Failed to find image %s in images. "
                            "Got images %s" % (self.image1_id, images))
        self.assertNotEmpty([i for i in images
                             if i['id'] == self.snapshot2_id])
        self.assertEmpty([i for i in images if i['id'] == self.snapshot3_id])

    @decorators.idempotent_id('05a377b8-28cf-4734-a1e6-2ab5c38bf606')
    @testtools.skipUnless(CONF.compute_feature_enabled.snapshot,
                          'Snapshotting is not available.')
    def test_list_images_filter_by_server_ref(self):
        """Test listing images filtered by server link href

        The list of images should contain only images with the
        provided server link href.
        """
        server_links = self.server2['links']

        # Try all server link types
        for link in server_links:
            params = {'server': link['href']}
            images = self.client.list_images(**params)['images']

            self.assertEmpty([i for i in images
                              if i['id'] == self.snapshot1_id])
            self.assertEmpty([i for i in images
                              if i['id'] == self.snapshot2_id])
            self.assertNotEmpty([i for i in images
                                 if i['id'] == self.snapshot3_id])

    @decorators.idempotent_id('e3356918-4d3e-4756-81d5-abc4524ba29f')
    @testtools.skipUnless(CONF.compute_feature_enabled.snapshot,
                          'Snapshotting is not available.')
    def test_list_images_filter_by_type(self):
        """Test listing images filtered by image type

        The list of images should contain only images with the
        provided image type.
        """
        params = {'type': 'snapshot'}
        images = self.client.list_images(**params)['images']

        self.assertNotEmpty([i for i in images
                             if i['id'] == self.snapshot1_id])
        self.assertNotEmpty([i for i in images
                             if i['id'] == self.snapshot2_id])
        self.assertNotEmpty([i for i in images
                             if i['id'] == self.snapshot3_id])
        self.assertEmpty([i for i in images if i['id'] == self.image_ref])

    @decorators.idempotent_id('3a484ca9-67ba-451e-b494-7fcf28d32d62')
    def test_list_images_limit_results(self):
        """Test listing images with limited count

        If we use limit=1 when listing images, then only 1 image should be
        returned.
        """
        params = {'limit': '1'}
        images = self.client.list_images(**params)['images']
        self.assertEqual(1, len([x for x in images if 'id' in x]))

    @decorators.idempotent_id('18bac3ae-da27-436c-92a9-b22474d13aab')
    def test_list_images_filter_by_changes_since(self):
        """Test listing images filtered by changes-since

        The list of images should contain only images updated since the
        provided changes-since value.
        """
        # Verify only updated images are returned in the detailed list

        # Becoming ACTIVE will modify the updated time
        # Filter by the image's created time
        params = {'changes-since': self.image3['created']}
        images = self.client.list_images(**params)['images']
        found = [i for i in images if i['id'] == self.image3_id]
        self.assertNotEmpty(found)

    @decorators.idempotent_id('9b0ea018-6185-4f71-948a-a123a107988e')
    def test_list_images_with_detail_filter_by_status(self):
        """Test listing server images details filtered by image status

        The list of images should contain only images with the
        provided image status.
        """
        params = {'status': 'ACTIVE'}
        images = self.client.list_images(detail=True, **params)['images']

        self.assertNotEmpty([i for i in images if i['id'] == self.image1_id])
        self.assertNotEmpty([i for i in images if i['id'] == self.image2_id])
        self.assertNotEmpty([i for i in images if i['id'] == self.image3_id])

    @decorators.idempotent_id('644ea267-9bd9-4f3b-af9f-dffa02396a17')
    def test_list_images_with_detail_filter_by_name(self):
        """Test listing server images details filtered by image name

        The list of images should contain only images with the
        provided image name.
        """
        params = {'name': self.image1['name']}
        images = self.client.list_images(detail=True, **params)['images']

        self.assertNotEmpty([i for i in images if i['id'] == self.image1_id])
        self.assertEmpty([i for i in images if i['id'] == self.image2_id])
        self.assertEmpty([i for i in images if i['id'] == self.image3_id])

    @decorators.idempotent_id('ba2fa9a9-b672-47cc-b354-3b4c0600e2cb')
    def test_list_images_with_detail_limit_results(self):
        """Test listing images details with limited count

        If we use limit=1 when listing images with full details, then only 1
        image should be returned.
        """
        params = {'limit': '1'}
        images = self.client.list_images(detail=True, **params)['images']
        self.assertEqual(1, len(images))

    @decorators.idempotent_id('8c78f822-203b-4bf6-8bba-56ebd551cf84')
    @testtools.skipUnless(CONF.compute_feature_enabled.snapshot,
                          'Snapshotting is not available.')
    def test_list_images_with_detail_filter_by_server_ref(self):
        """Test listing images details filtered by server link href

        The list of images should contain only images with the
        provided server link href.
        """
        server_links = self.server2['links']

        # Try all server link types
        for link in server_links:
            params = {'server': link['href']}
            images = self.client.list_images(detail=True, **params)['images']

            self.assertEmpty([i for i in images
                              if i['id'] == self.snapshot1_id])
            self.assertEmpty([i for i in images
                              if i['id'] == self.snapshot2_id])
            self.assertNotEmpty([i for i in images
                                 if i['id'] == self.snapshot3_id])

    @decorators.idempotent_id('888c0cc0-7223-43c5-9db0-b125fd0a393b')
    @testtools.skipUnless(CONF.compute_feature_enabled.snapshot,
                          'Snapshotting is not available.')
    def test_list_images_with_detail_filter_by_type(self):
        """Test listing images details filtered by image type

        The list of images should contain only images with the
        provided image type.
        """
        params = {'type': 'snapshot'}
        images = self.client.list_images(detail=True, **params)['images']
        self.client.show_image(self.image_ref)

        self.assertNotEmpty([i for i in images
                             if i['id'] == self.snapshot1_id])
        self.assertNotEmpty([i for i in images
                             if i['id'] == self.snapshot2_id])
        self.assertNotEmpty([i for i in images
                             if i['id'] == self.snapshot3_id])
        self.assertEmpty([i for i in images if i['id'] == self.image_ref])

    @decorators.idempotent_id('7d439e18-ac2e-4827-b049-7e18004712c4')
    def test_list_images_with_detail_filter_by_changes_since(self):
        """Test listing images details filtered by changes-since

        The list of images should contain only images updated since the
        provided changes-since value.
        """
        # Becoming ACTIVE will modify the updated time
        # Filter by the image's created time
        params = {'changes-since': self.image1['created']}
        images = self.client.list_images(detail=True, **params)['images']
        self.assertNotEmpty([i for i in images if i['id'] == self.image1_id])



# Copyright 2016 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import io

from tempest.lib.common.utils import data_utils
from tempest.lib.services.image.v2 import images_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestImagesClient(base.BaseServiceTest):
    FAKE_CREATE_UPDATE_SHOW_IMAGE = {
        "id": "e485aab9-0907-4973-921c-bb6da8a8fcf8",
        "name": u"\u2740(*\xb4\u25e2`*)\u2740",
        "status": "active",
        "visibility": "public",
        "size": 2254249,
        "checksum": "2cec138d7dae2aa59038ef8c9aec2390",
        "tags": [
            "fedora",
            "beefy"
        ],
        "created_at": "2012-08-10T19:23:50Z",
        "updated_at": "2012-08-12T11:11:33Z",
        "self": "/v2/images/da3b75d9-3f4a-40e7-8a2c-bfab23927dea",
        "file": "/v2/images/da3b75d9-3f4a-40e7-8a2c-bfab23927"
                "dea/file",
        "schema": "/v2/schemas/image",
        "owner": None,
        "min_ram": None,
        "min_disk": None,
        "disk_format": None,
        "virtual_size": None,
        "container_format": None,
        "os_hash_algo": "sha512",
        "os_hash_value": "ef7d1ed957ffafefb324d50ebc6685ed03d0e645d",
        "os_hidden": False,
        "protected": False,
    }

    FAKE_LIST_IMAGES = {
        "images": [
            {
                "status": "active",
                "name": "cirros-0.3.2-x86_64-disk",
                "tags": [],
                "container_format": "bare",
                "created_at": "2014-11-07T17:07:06Z",
                "disk_format": "qcow2",
                "updated_at": "2014-11-07T17:19:09Z",
                "visibility": "public",
                "self": "/v2/images/1bea47ed-f6a9-463b-b423-14b9cca9ad27",
                "min_disk": 0,
                "protected": False,
                "id": "1bea47ed-f6a9-463b-b423-14b9cca9ad27",
                "file": "/v2/images/1bea47ed-f6a9-463b-b423-14b9cca9ad27/file",
                "checksum": "64d7c1cd2b6f60c92c14662941cb7913",
                "owner": "5ef70662f8b34079a6eddb8da9d75fe8",
                "size": 13167616,
                "min_ram": 0,
                "schema": "/v2/schemas/image",
                "virtual_size": None,
                "os_hash_algo": "sha512",
                "os_hash_value": "ef7d1ed957ffafefb324d50ebc6685ed03d0e645d",
                "os_hidden": False
            },
            {
                "status": "active",
                "name": "F17-x86_64-cfntools",
                "tags": [],
                "container_format": "bare",
                "created_at": "2014-10-30T08:23:39Z",
                "disk_format": "qcow2",
                "updated_at": "2014-11-03T16:40:10Z",
                "visibility": "public",
                "self": "/v2/images/781b3762-9469-4cec-b58d-3349e5de4e9c",
                "min_disk": 0,
                "protected": False,
                "id": "781b3762-9469-4cec-b58d-3349e5de4e9c",
                "file": "/v2/images/781b3762-9469-4cec-b58d-3349e5de4e9c/file",
                "checksum": "afab0f79bac770d61d24b4d0560b5f70",
                "owner": "5ef70662f8b34079a6eddb8da9d75fe8",
                "size": 476704768,
                "min_ram": 0,
                "schema": "/v2/schemas/image",
                "virtual_size": None,
                "os_hash_algo": "sha512",
                "os_hash_value": "ef7d1ed957ffafefb324d50ebc6685ed03d0e645d",
                "os_hidden": False
            }
        ],
        "schema": "/v2/schemas/images",
        "first": "/v2/images"
    }

    FAKE_SHOW_IMAGE_TASKS = {
        "tasks": [
            {
                "id": "ee22890e-8948-4ea6-9668-831f973c84f5",
                "image_id": "dddddddd-dddd-dddd-dddd-dddddddddddd",
                "request-id": "rrrrrrr-rrrr-rrrr-rrrr-rrrrrrrrrrrr",
                "user": "uuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuu",
                "type": "api_image_import",
                "status": "processing",
                "owner": "64f0efc9955145aeb06f297a8a6fe402",
                "expires_at": None,
                "created_at": "2020-12-18T05:20:38.000000",
                "updated_at": "2020-12-18T05:25:39.000000",
                "deleted_at": None,
                "deleted": False,
                "input": {
                    "image_id": "829c729b-ebc4-4cc7-a164-6f43f1149b17",
                    "import_req": {
                        "method": {
                            "name": "copy-image",
                        },
                        "all_stores": True,
                        "all_stores_must_succeed": False,
                    },
                    "backend": [
                        "fast",
                        "cheap",
                        "slow",
                        "reliable",
                        "common",
                    ]
                },
                "result": None,
                "message": "Copied 15 MiB",
            }
        ]
    }

    FAKE_TAG_NAME = "fake tag"

    def setUp(self):
        super(TestImagesClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.client = images_client.ImagesClient(fake_auth,
                                                 'image', 'regionOne')

    def _test_update_image(self, bytes_body=False):
        self.check_service_client_function(
            self.client.update_image,
            'tempest.lib.common.rest_client.RestClient.patch',
            self.FAKE_CREATE_UPDATE_SHOW_IMAGE,
            bytes_body,
            image_id="e485aab9-0907-4973-921c-bb6da8a8fcf8",
            patch=[{"op": "add", "path": "/a/b/c", "value": ["foo", "bar"]}])

    def _test_create_image(self, bytes_body=False):
        self.check_service_client_function(
            self.client.create_image,
            'tempest.lib.common.rest_client.RestClient.post',
            self.FAKE_CREATE_UPDATE_SHOW_IMAGE,
            bytes_body,
            name="virtual machine image",
            status=201)

    def _test_show_image(self, bytes_body=False):
        self.check_service_client_function(
            self.client.show_image,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_CREATE_UPDATE_SHOW_IMAGE,
            bytes_body,
            image_id="e485aab9-0907-4973-921c-bb6da8a8fcf8")

    def _test_list_images(self, bytes_body=False):
        self.check_service_client_function(
            self.client.list_images,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_LIST_IMAGES,
            bytes_body,
            mock_args=['images'])

    def test_create_image_with_str_body(self):
        self._test_create_image()

    def test_create_image_with_bytes_body(self):
        self._test_create_image(bytes_body=True)

    def test_update_image_with_str_body(self):
        self._test_update_image()

    def test_update_image_with_bytes_body(self):
        self._test_update_image(bytes_body=True)

    def test_deactivate_image(self):
        self.check_service_client_function(
            self.client.deactivate_image,
            'tempest.lib.common.rest_client.RestClient.post',
            {}, image_id="e485aab9-0907-4973-921c-bb6da8a8fcf8", status=204)

    def test_reactivate_image(self):
        self.check_service_client_function(
            self.client.reactivate_image,
            'tempest.lib.common.rest_client.RestClient.post',
            {}, image_id="e485aab9-0907-4973-921c-bb6da8a8fcf8", status=204)

    def test_delete_image(self):
        self.check_service_client_function(
            self.client.delete_image,
            'tempest.lib.common.rest_client.RestClient.delete',
            {}, image_id="e485aab9-0907-4973-921c-bb6da8a8fcf8", status=204)

    def test_store_image_file(self):
        data = io.BytesIO(data_utils.random_bytes())

        self.check_service_client_function(
            self.client.store_image_file,
            'tempest.lib.common.rest_client.RestClient.raw_request',
            {},
            image_id=self.FAKE_CREATE_UPDATE_SHOW_IMAGE["id"],
            status=204,
            data=data)

    def test_show_image_file(self):
        # NOTE: The response for this API returns raw binary data, but an error
        # is thrown if random bytes are used for the resp body since
        # ``create_response`` then calls ``json.dumps``.
        self.check_service_client_function(
            self.client.show_image_file,
            'tempest.lib.common.rest_client.RestClient.get',
            {},
            resp_as_string=True,
            image_id=self.FAKE_CREATE_UPDATE_SHOW_IMAGE["id"],
            headers={'Content-Type': 'application/octet-stream'},
            status=200)

    def test_add_image_tag(self):
        self.check_service_client_function(
            self.client.add_image_tag,
            'tempest.lib.common.rest_client.RestClient.put',
            {},
            image_id=self.FAKE_CREATE_UPDATE_SHOW_IMAGE["id"],
            status=204,
            tag=self.FAKE_TAG_NAME)

    def test_delete_image_tag(self):
        self.check_service_client_function(
            self.client.delete_image_tag,
            'tempest.lib.common.rest_client.RestClient.delete',
            {},
            image_id=self.FAKE_CREATE_UPDATE_SHOW_IMAGE["id"],
            status=204,
            tag=self.FAKE_TAG_NAME)

    def test_show_image_with_str_body(self):
        self._test_show_image()

    def test_show_image_with_bytes_body(self):
        self._test_show_image(bytes_body=True)

    def test_list_images_with_str_body(self):
        self._test_list_images()

    def test_list_images_with_bytes_body(self):
        self._test_list_images(bytes_body=True)

    def test_show_image_tasks(self):
        self.check_service_client_function(
            self.client.show_image_tasks,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_SHOW_IMAGE_TASKS,
            True,
            image_id="e485aab9-0907-4973-921c-bb6da8a8fcf8")



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.image import base
from tempest.lib import decorators


class ImagesMemberTest(base.BaseV2MemberImageTest):
    """Test image members"""

    @decorators.idempotent_id('5934c6ea-27dc-4d6e-9421-eeb5e045494a')
    def test_image_share_accept(self):
        """Test sharing and accepting an image"""
        image_id = self._create_image()
        member = self.image_member_client.create_image_member(
            image_id, member=self.alt_tenant_id)
        self.addCleanup(self.image_member_client.delete_image_member,
                        image_id, self.alt_tenant_id)
        self.assertEqual(member['member_id'], self.alt_tenant_id)
        self.assertEqual(member['image_id'], image_id)
        self.assertEqual(member['status'], 'pending')
        self.assertNotIn(image_id, self._list_image_ids_as_alt())
        self.alt_image_member_client.update_image_member(image_id,
                                                         self.alt_tenant_id,
                                                         status='accepted')
        self.assertIn(image_id, self._list_image_ids_as_alt())
        body = self.image_member_client.list_image_members(image_id)
        members = body['members']
        self.assertEqual(len(members), 1, str(members))
        member = members[0]
        self.assertEqual(member['member_id'], self.alt_tenant_id)
        self.assertEqual(member['image_id'], image_id)
        self.assertEqual(member['status'], 'accepted')

    @decorators.idempotent_id('d9e83e5f-3524-4b38-a900-22abcb26e90e')
    def test_image_share_reject(self):
        """Test sharing and rejecting an image"""
        image_id = self._create_image()
        member = self.image_member_client.create_image_member(
            image_id, member=self.alt_tenant_id)
        self.addCleanup(self.image_member_client.delete_image_member,
                        image_id, self.alt_tenant_id)
        self.assertEqual(member['member_id'], self.alt_tenant_id)
        self.assertEqual(member['image_id'], image_id)
        self.assertEqual(member['status'], 'pending')
        self.assertNotIn(image_id, self._list_image_ids_as_alt())
        self.alt_image_member_client.update_image_member(image_id,
                                                         self.alt_tenant_id,
                                                         status='rejected')
        self.assertNotIn(image_id, self._list_image_ids_as_alt())

    @decorators.idempotent_id('a6ee18b9-4378-465e-9ad9-9a6de58a3287')
    def test_get_image_member(self):
        """Test getting image members after the image is accepted"""
        image_id = self._create_image()
        self.image_member_client.create_image_member(
            image_id, member=self.alt_tenant_id)
        self.addCleanup(self.image_member_client.delete_image_member,
                        image_id, self.alt_tenant_id)
        self.alt_image_member_client.update_image_member(image_id,
                                                         self.alt_tenant_id,
                                                         status='accepted')

        self.assertIn(image_id, self._list_image_ids_as_alt())
        member = self.image_member_client.show_image_member(
            image_id, self.alt_tenant_id)
        self.assertEqual(self.alt_tenant_id, member['member_id'])
        self.assertEqual(image_id, member['image_id'])
        self.assertEqual('accepted', member['status'])

    @decorators.idempotent_id('72989bc7-2268-48ed-af22-8821e835c914')
    def test_remove_image_member(self):
        """Test removing image members after the image is accepted"""
        image_id = self._create_image()
        self.image_member_client.create_image_member(
            image_id, member=self.alt_tenant_id)
        self.alt_image_member_client.update_image_member(image_id,
                                                         self.alt_tenant_id,
                                                         status='accepted')

        self.assertIn(image_id, self._list_image_ids_as_alt())
        self.image_member_client.delete_image_member(image_id,
                                                     self.alt_tenant_id)
        self.assertNotIn(image_id, self._list_image_ids_as_alt())

    @decorators.idempotent_id('634dcc3f-f6e2-4409-b8fd-354a0bb25d83')
    def test_get_image_member_schema(self):
        """Test getting image member schema"""
        body = self.schemas_client.show_schema("member")
        self.assertEqual("member", body['name'])

    @decorators.idempotent_id('6ae916ef-1052-4e11-8d36-b3ae14853cbb')
    def test_get_image_members_schema(self):
        """Test getting image members schema"""
        body = self.schemas_client.show_schema("members")
        self.assertEqual("members", body['name'])



# Copyright 2017 AT&T Corp.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import six

from tempest.api.compute import base
from tempest.common import utils
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators


class ServerTagsTestJSON(base.BaseV2ComputeTest):
    """Test server tags with compute microversion greater than 2.25"""

    min_microversion = '2.26'
    max_microversion = 'latest'

    create_default_network = True

    @classmethod
    def skip_checks(cls):
        super(ServerTagsTestJSON, cls).skip_checks()
        if not utils.is_extension_enabled('os-server-tags', 'compute'):
            msg = "os-server-tags extension is not enabled."
            raise cls.skipException(msg)

    @classmethod
    def setup_clients(cls):
        super(ServerTagsTestJSON, cls).setup_clients()
        cls.client = cls.servers_client

    @classmethod
    def resource_setup(cls):
        super(ServerTagsTestJSON, cls).resource_setup()
        cls.server = cls.create_test_server(wait_until='ACTIVE')

    def _update_server_tags(self, server_id, tags):
        if not isinstance(tags, (list, tuple)):
            tags = [tags]
        for tag in tags:
            self.client.update_tag(server_id, tag)
        self.addCleanup(self.client.delete_all_tags, server_id)

    @decorators.idempotent_id('8d95abe2-c658-4c42-9a44-c0258500306b')
    def test_create_delete_tag(self):
        """Test creating and deleting server tag"""
        # Check that no tags exist.
        fetched_tags = self.client.list_tags(self.server['id'])['tags']
        self.assertEmpty(fetched_tags)

        # Add server tag to the server.
        assigned_tag = data_utils.rand_name('tag')
        self._update_server_tags(self.server['id'], assigned_tag)

        # Check that added tag exists.
        fetched_tags = self.client.list_tags(self.server['id'])['tags']
        self.assertEqual([assigned_tag], fetched_tags)

        # Remove assigned tag from server and check that it was removed.
        self.client.delete_tag(self.server['id'], assigned_tag)
        fetched_tags = self.client.list_tags(self.server['id'])['tags']
        self.assertEmpty(fetched_tags)

    @decorators.idempotent_id('a2c1af8c-127d-417d-974b-8115f7e3d831')
    def test_update_all_tags(self):
        """Test updating all server tags"""
        # Add server tags to the server.
        tags = [data_utils.rand_name('tag'), data_utils.rand_name('tag')]
        self._update_server_tags(self.server['id'], tags)

        # Replace tags with new tags and check that they are present.
        new_tags = [data_utils.rand_name('tag'), data_utils.rand_name('tag')]
        replaced_tags = self.client.update_all_tags(
            self.server['id'], new_tags)['tags']
        six.assertCountEqual(self, new_tags, replaced_tags)

        # List the tags and check that the tags were replaced.
        fetched_tags = self.client.list_tags(self.server['id'])['tags']
        six.assertCountEqual(self, new_tags, fetched_tags)

    @decorators.idempotent_id('a63b2a74-e918-4b7c-bcab-10c855f3a57e')
    def test_delete_all_tags(self):
        """Test deleting all server tags"""
        # Add server tags to the server.
        assigned_tags = [data_utils.rand_name('tag'),
                         data_utils.rand_name('tag')]
        self._update_server_tags(self.server['id'], assigned_tags)

        # Delete tags from the server and check that they were deleted.
        self.client.delete_all_tags(self.server['id'])
        fetched_tags = self.client.list_tags(self.server['id'])['tags']
        self.assertEmpty(fetched_tags)

    @decorators.idempotent_id('81279a66-61c3-4759-b830-a2dbe64cbe08')
    def test_check_tag_existence(self):
        """Test checking server tag existence"""
        # Add server tag to the server.
        assigned_tag = data_utils.rand_name('tag')
        self._update_server_tags(self.server['id'], assigned_tag)

        # Check that added tag exists. Throws a 404 if not found, else a 204,
        # which was already checked by the schema validation.
        self.client.check_tag_existence(self.server['id'], assigned_tag)



# Copyright 2014 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

links = {
    'type': 'array',
    'items': {
        'type': 'object',
        'properties': {
            'href': {
                'type': 'string',
                'format': 'uri'
            },
            'rel': {'type': 'string'}
        },
        'additionalProperties': False,
        'required': ['href', 'rel']
    }
}

mac_address = {
    'type': 'string',
    'pattern': '(?:[a-f0-9]{2}:){5}[a-f0-9]{2}'
}

ip_address = {
    'oneOf': [
        {
            'type': 'string',
            'oneOf': [
                {'format': 'ipv4'},
                {'format': 'ipv6'}
            ]
        },
        {'type': 'null'}
    ]
}

access_ip_v4 = {
    'type': 'string',
    'oneOf': [{'format': 'ipv4'}, {'enum': ['']}]
}

access_ip_v6 = {
    'type': 'string',
    'oneOf': [{'format': 'ipv6'}, {'enum': ['']}]
}

addresses = {
    'type': 'object',
    'patternProperties': {
        # NOTE: Here is for 'private' or something.
        '^[a-zA-Z0-9-_.]+$': {
            'type': 'array',
            'items': {
                'type': 'object',
                'properties': {
                    'version': {'enum': [4, 6]},
                    'addr': {
                        'type': 'string',
                        'oneOf': [
                            {'format': 'ipv4'},
                            {'format': 'ipv6'}
                        ]
                    }
                },
                'additionalProperties': False,
                'required': ['version', 'addr']
            }
        }
    }
}

date_time = {
    'type': 'string',
    'format': 'iso8601-date-time'
}

date_time_or_null = {
    'type': ['string', 'null'],
    'format': 'iso8601-date-time'
}

response_header = {
    'connection': {'type': 'string'},
    'content-length': {'type': 'string'},
    'content-type': {'type': 'string'},
    'status': {'type': 'string'},
    'x-compute-request-id': {'type': 'string'},
    'vary': {'type': 'string'},
    'x-openstack-nova-api-version': {'type': 'string'},
    # NOTE(gmann): Validating this as string only as this
    # date in header is returned in different format than
    # ISO 8601 date time format which is not consistent with
    # other date-time format in nova.
    # This API is already deprecated so not worth to fix
    # on nova side.
    'date': {
        'type': 'string'
    }
}

power_state = {
    'type': 'integer',
    # 0: NOSTATE
    # 1: RUNNING
    # 3: PAUSED
    # 4: SHUTDOWN
    # 6: CRASHED
    # 7: SUSPENDED
    'enum': [0, 1, 3, 4, 6, 7]
}

uuid_or_null = {
    'anyOf': [
        {'type': 'string', 'format': 'uuid'},
        {'type': 'null'}
    ]
}



# Copyright 2013 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.compute import base
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc


class ServicesAdminNegativeTestJSON(base.BaseV2ComputeAdminTest):
    """Tests Services API. List and Enable/Disable require admin privileges."""
    max_microversion = '2.52'

    @classmethod
    def setup_clients(cls):
        super(ServicesAdminNegativeTestJSON, cls).setup_clients()
        cls.client = cls.os_admin.services_client
        cls.non_admin_client = cls.services_client

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('1126d1f8-266e-485f-a687-adc547492646')
    def test_list_services_with_non_admin_user(self):
        """Non admin user is not allowed to list nova services"""
        self.assertRaises(lib_exc.Forbidden,
                          self.non_admin_client.list_services)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('d0884a69-f693-4e79-a9af-232d15643bf7')
    def test_get_service_by_invalid_params(self):
        """Test listing services by invalid filter should return all services

        Expect all services to be returned when the request contains invalid
        parameters.
        """
        if not self.is_requested_microversion_compatible('2.74'):
            raise self.skipException(
                "From microversion 2.75 invalid parameters are not allowed.")
        services = self.client.list_services()['services']
        services_xxx = (self.client.list_services(xxx='nova-compute')
                        ['services'])
        self.assertEqual(len(services), len(services_xxx))

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('1e966d4a-226e-47c7-b601-0b18a27add54')
    def test_get_service_by_invalid_service_and_valid_host(self):
        """Test listing services by invalid service and valid host value"""
        services = self.client.list_services()['services']
        host_name = services[0]['host']
        services = self.client.list_services(host=host_name,
                                             binary='xxx')['services']
        self.assertEmpty(services)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('64e7e7fb-69e8-4cb6-a71d-8d5eb0c98655')
    def test_get_service_with_valid_service_and_invalid_host(self):
        """Test listing services by valid service and invalid host value"""
        services = self.client.list_services()['services']
        binary_name = services[0]['binary']
        services = self.client.list_services(host='xxx',
                                             binary=binary_name)['services']
        self.assertEmpty(services)


class ServicesAdminNegativeV253TestJSON(ServicesAdminNegativeTestJSON):
    min_microversion = '2.53'
    max_microversion = 'latest'

    # NOTE(felipemonteiro): This class tests the services APIs response schema
    # for the 2.53 microversion. Schema testing is done for `list_services`
    # tests.

    @classmethod
    def resource_setup(cls):
        super(ServicesAdminNegativeV253TestJSON, cls).resource_setup()
        cls.fake_service_id = data_utils.rand_uuid()

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('508671aa-c929-4479-bd10-8680d40dd0a6')
    def test_enable_service_with_invalid_service_id(self):
        """Test updating non existing service to status enabled"""
        self.assertRaises(lib_exc.NotFound,
                          self.client.update_service,
                          service_id=self.fake_service_id,
                          status='enabled')

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('a9eeeade-42b3-419f-87aa-c9342aa068cf')
    def test_disable_service_with_invalid_service_id(self):
        """Test updating non existing service to status disabled"""
        self.assertRaises(lib_exc.NotFound,
                          self.client.update_service,
                          service_id=self.fake_service_id,
                          status='disabled')

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('f46a9d91-1e85-4b96-8e7a-db7706fa2e9a')
    def test_disable_log_reason_with_invalid_service_id(self):
        """Test updating non existing service to disabled with reason"""

        # disabled_reason requires that status='disabled' be provided.
        self.assertRaises(lib_exc.NotFound,
                          self.client.update_service,
                          service_id=self.fake_service_id,
                          status='disabled',
                          disabled_reason='maintenance')



# Copyright 2014 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy

from tempest.lib.api_schema.response.compute.v2_1 import parameter_types

# create-aggregate api doesn't have 'hosts' and 'metadata' attributes.
aggregate_for_create = {
    'type': 'object',
    'properties': {
        'availability_zone': {'type': ['string', 'null']},
        'created_at': parameter_types.date_time,
        'deleted': {'type': 'boolean'},
        'deleted_at': parameter_types.date_time_or_null,
        'id': {'type': 'integer'},
        'name': {'type': 'string'},
        'updated_at': parameter_types.date_time_or_null
    },
    'additionalProperties': False,
    'required': ['availability_zone', 'created_at', 'deleted',
                 'deleted_at', 'id', 'name', 'updated_at'],
}

common_aggregate_info = copy.deepcopy(aggregate_for_create)
common_aggregate_info['properties'].update({
    'hosts': {'type': 'array'},
    'metadata': {'type': 'object'}
})
common_aggregate_info['required'].extend(['hosts', 'metadata'])

list_aggregates = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'aggregates': {
                'type': 'array',
                'items': common_aggregate_info
            }
        },
        'additionalProperties': False,
        'required': ['aggregates'],
    }
}

get_aggregate = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'aggregate': common_aggregate_info
        },
        'additionalProperties': False,
        'required': ['aggregate'],
    }
}

aggregate_set_metadata = get_aggregate
# The 'updated_at' attribute of 'update_aggregate' can't be null.
update_aggregate = copy.deepcopy(get_aggregate)
update_aggregate['response_body']['properties']['aggregate']['properties'][
    'updated_at'] = parameter_types.date_time

delete_aggregate = {
    'status_code': [200]
}

create_aggregate = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'aggregate': aggregate_for_create
        },
        'additionalProperties': False,
        'required': ['aggregate'],
    }
}

aggregate_add_remove_host = get_aggregate



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.compute import base
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators

CONF = config.CONF


class ImagesOneServerTestJSON(base.BaseV2ComputeTest):
    """Test server images API"""

    @classmethod
    def resource_setup(cls):
        super(ImagesOneServerTestJSON, cls).resource_setup()
        cls.server_id = cls.create_test_server(wait_until='ACTIVE')['id']

    @classmethod
    def skip_checks(cls):
        super(ImagesOneServerTestJSON, cls).skip_checks()
        if not CONF.service_available.glance:
            skip_msg = ("%s skipped as glance is not available" % cls.__name__)
            raise cls.skipException(skip_msg)

        if not CONF.compute_feature_enabled.snapshot:
            skip_msg = ("%s skipped as instance snapshotting is not supported"
                        % cls.__name__)
            raise cls.skipException(skip_msg)

    @classmethod
    def setup_clients(cls):
        super(ImagesOneServerTestJSON, cls).setup_clients()
        if cls.is_requested_microversion_compatible('2.35'):
            cls.client = cls.compute_images_client
        else:
            cls.client = cls.images_client

    def _get_default_flavor_disk_size(self, flavor_id):
        flavor = self.flavors_client.show_flavor(flavor_id)['flavor']
        return flavor['disk']

    @decorators.idempotent_id('3731d080-d4c5-4872-b41a-64d0d0021314')
    def test_create_delete_image(self):
        """Test create/delete server image"""
        if self.is_requested_microversion_compatible('2.35'):
            MIN_DISK = 'minDisk'
            MIN_RAM = 'minRam'
        else:
            MIN_DISK = 'min_disk'
            MIN_RAM = 'min_ram'

        # Create a new image
        name = data_utils.rand_name('image')
        meta = {'image_type': 'test'}
        image = self.create_image_from_server(self.server_id, name=name,
                                              metadata=meta,
                                              wait_until='ACTIVE')

        # Verify the image was created correctly
        self.assertEqual(name, image['name'])
        if self.is_requested_microversion_compatible('2.35'):
            self.assertEqual('test', image['metadata']['image_type'])
        else:
            self.assertEqual('test', image['image_type'])

        original_image = self.client.show_image(self.image_ref)
        if self.is_requested_microversion_compatible('2.35'):
            original_image = original_image['image']

        # Verify minRAM is the same as the original image
        self.assertEqual(image[MIN_RAM], original_image[MIN_RAM])

        # Verify minDisk is the same as the original image or the flavor size
        flavor_disk_size = self._get_default_flavor_disk_size(self.flavor_ref)
        self.assertIn(str(image[MIN_DISK]),
                      (str(original_image[MIN_DISK]), str(flavor_disk_size)))

        # Verify the image was deleted correctly
        self.client.delete_image(image['id'])
        self.client.wait_for_resource_deletion(image['id'])

    @decorators.idempotent_id('3b7c6fe4-dfe7-477c-9243-b06359db51e6')
    def test_create_image_specify_multibyte_character_image_name(self):
        """Test creating server image with multibyte character image name"""
        # prefix character is:
        # http://unicode.org/cldr/utility/character.jsp?a=20A1

        # We use a string with 3 byte utf-8 character due to nova/glance which
        # will return 400(Bad Request) if we attempt to send a name which has
        # 4 byte utf-8 character.
        utf8_name = data_utils.rand_name(b'\xe2\x82\xa1'.decode('utf-8'))
        self.create_image_from_server(self.server_id, name=utf8_name,
                                      wait_until='ACTIVE')



# Copyright 2015 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.services.compute import aggregates_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestAggregatesClient(base.BaseServiceTest):
    FAKE_SHOW_AGGREGATE = {
        "aggregate":
        {
            "name": "hoge",
            "availability_zone": None,
            "deleted": False,
            "created_at":
            "2015-07-16T03:07:32.000000",
            "updated_at": None,
            "hosts": [],
            "deleted_at": None,
            "id": 1,
            "metadata": {}
        }
    }

    FAKE_CREATE_AGGREGATE = {
        "aggregate":
        {
            "name": u'\xf4',
            "availability_zone": None,
            "deleted": False,
            "created_at": "2015-07-21T04:11:18.000000",
            "updated_at": None,
            "deleted_at": None,
            "id": 1
        }
    }

    FAKE_UPDATE_AGGREGATE = {
        "aggregate":
        {
            "name": u'\xe9',
            "availability_zone": None,
            "deleted": False,
            "created_at": "2015-07-16T03:07:32.000000",
            "updated_at": "2015-07-23T05:16:29.000000",
            "hosts": [],
            "deleted_at": None,
            "id": 1,
            "metadata": {}
        }
    }

    FAKE_AGGREGATE = {
        "availability_zone": "nova",
        "created_at": "2013-08-18T12:17:56.297823",
        "deleted": False,
        "deleted_at": None,
        "hosts": [
            "21549b2f665945baaa7101926a00143c"
        ],
        "id": 1,
        "metadata": {
            "availability_zone": "nova"
        },
        "name": u'\xe9',
        "updated_at": None
    }

    FAKE_ADD_HOST = {'aggregate': FAKE_AGGREGATE}
    FAKE_REMOVE_HOST = {'aggregate': FAKE_AGGREGATE}
    FAKE_SET_METADATA = {'aggregate': FAKE_AGGREGATE}

    def setUp(self):
        super(TestAggregatesClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.client = aggregates_client.AggregatesClient(
            fake_auth, 'compute', 'regionOne')

    def _test_list_aggregates(self, bytes_body=False):
        self.check_service_client_function(
            self.client.list_aggregates,
            'tempest.lib.common.rest_client.RestClient.get',
            {"aggregates": []},
            bytes_body)

    def test_list_aggregates_with_str_body(self):
        self._test_list_aggregates()

    def test_list_aggregates_with_bytes_body(self):
        self._test_list_aggregates(bytes_body=True)

    def _test_show_aggregate(self, bytes_body=False):
        self.check_service_client_function(
            self.client.show_aggregate,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_SHOW_AGGREGATE,
            bytes_body,
            aggregate_id=1)

    def test_show_aggregate_with_str_body(self):
        self._test_show_aggregate()

    def test_show_aggregate_with_bytes_body(self):
        self._test_show_aggregate(bytes_body=True)

    def _test_create_aggregate(self, bytes_body=False):
        self.check_service_client_function(
            self.client.create_aggregate,
            'tempest.lib.common.rest_client.RestClient.post',
            self.FAKE_CREATE_AGGREGATE,
            bytes_body,
            name='hoge')

    def test_create_aggregate_with_str_body(self):
        self._test_create_aggregate()

    def test_create_aggregate_with_bytes_body(self):
        self._test_create_aggregate(bytes_body=True)

    def test_delete_aggregate(self):
        self.check_service_client_function(
            self.client.delete_aggregate,
            'tempest.lib.common.rest_client.RestClient.delete',
            {}, aggregate_id="1")

    def _test_update_aggregate(self, bytes_body=False):
        self.check_service_client_function(
            self.client.update_aggregate,
            'tempest.lib.common.rest_client.RestClient.put',
            self.FAKE_UPDATE_AGGREGATE,
            bytes_body,
            aggregate_id=1)

    def test_update_aggregate_with_str_body(self):
        self._test_update_aggregate()

    def test_update_aggregate_with_bytes_body(self):
        self._test_update_aggregate(bytes_body=True)

    def _test_add_host(self, bytes_body=False):
        self.check_service_client_function(
            self.client.add_host,
            'tempest.lib.common.rest_client.RestClient.post',
            self.FAKE_ADD_HOST,
            bytes_body,
            aggregate_id=1)

    def test_add_host_with_str_body(self):
        self._test_add_host()

    def test_add_host_with_bytes_body(self):
        self._test_add_host(bytes_body=True)

    def _test_remove_host(self, bytes_body=False):
        self.check_service_client_function(
            self.client.remove_host,
            'tempest.lib.common.rest_client.RestClient.post',
            self.FAKE_REMOVE_HOST,
            bytes_body,
            aggregate_id=1)

    def test_remove_host_with_str_body(self):
        self._test_remove_host()

    def test_remove_host_with_bytes_body(self):
        self._test_remove_host(bytes_body=True)

    def _test_set_metadata(self, bytes_body=False):
        self.check_service_client_function(
            self.client.set_metadata,
            'tempest.lib.common.rest_client.RestClient.post',
            self.FAKE_SET_METADATA,
            bytes_body,
            aggregate_id=1)

    def test_set_metadata_with_str_body(self):
        self._test_set_metadata()

    def test_set_metadata_with_bytes_body(self):
        self._test_set_metadata(bytes_body=True)



# Copyright 2015 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_log import log as logging
from oslo_serialization import jsonutils as json

from tempest.lib.common import rest_client
from tempest.lib import exceptions


class V3TokenClient(rest_client.RestClient):

    def __init__(self, auth_url, disable_ssl_certificate_validation=None,
                 ca_certs=None, trace_requests=None, **kwargs):
        """Initialises the Token client

        :param auth_url: URL to which the token request is sent
        :param disable_ssl_certificate_validation: pass-through to rest client
        :param ca_certs: pass-through to rest client
        :param trace_requests: pass-through to rest client
        :param kwargs: any extra parameter to pass through the rest client.
               Three kwargs are forbidden: region, service and auth_provider
               as they are not meaningful for token client
        """
        dscv = disable_ssl_certificate_validation
        for unwanted_kwargs in ['region', 'service', 'auth_provider']:
            kwargs.pop(unwanted_kwargs, None)
        super(V3TokenClient, self).__init__(
            None, None, None, disable_ssl_certificate_validation=dscv,
            ca_certs=ca_certs, trace_requests=trace_requests, **kwargs)

        if auth_url is None:
            raise exceptions.IdentityError("Couldn't determine auth_url")

        if 'auth/tokens' not in auth_url:
            auth_url = auth_url.rstrip('/') + '/auth/tokens'

        self.auth_url = auth_url

    def auth(self, user_id=None, username=None, password=None, project_id=None,
             project_name=None, user_domain_id=None, user_domain_name=None,
             project_domain_id=None, project_domain_name=None, domain_id=None,
             domain_name=None, system=None, token=None, app_cred_id=None,
             app_cred_secret=None):
        """Obtains a token from the authentication service

        :param user_id: user id
        :param username: user name
        :param user_domain_id: the user domain id
        :param user_domain_name: the user domain name
        :param project_domain_id: the project domain id
        :param project_domain_name: the project domain name
        :param domain_id: a domain id to scope to
        :param domain_name: a domain name to scope to
        :param project_id: a project id to scope to
        :param project_name: a project name to scope to
        :param system: whether the token should be scoped to the system
        :param token: a token to re-scope.

        Accepts different combinations of credentials.
        Sample sample valid combinations:
        - token
        - token, project_name, project_domain_id
        - user_id, password
        - username, password, user_domain_id
        - username, password, project_name, user_domain_id, project_domain_id
        - username, password, user_domain_id, system
        Validation is left to the server side.
        """
        creds = {
            'auth': {
                'identity': {
                    'methods': [],
                }
            }
        }
        id_obj = creds['auth']['identity']
        if token:
            id_obj['methods'].append('token')
            id_obj['token'] = {
                'id': token
            }

        if (user_id or username) and password:
            id_obj['methods'].append('password')
            id_obj['password'] = {
                'user': {
                    'password': password,
                }
            }
            if user_id:
                id_obj['password']['user']['id'] = user_id
            else:
                id_obj['password']['user']['name'] = username

            _domain = None
            if user_domain_id is not None:
                _domain = dict(id=user_domain_id)
            elif user_domain_name is not None:
                _domain = dict(name=user_domain_name)
            if _domain:
                id_obj['password']['user']['domain'] = _domain

        if app_cred_id and app_cred_secret:
            id_obj['methods'].append('application_credential')
            id_obj['application_credential'] = {
                'id': app_cred_id,
                'secret': app_cred_secret,
            }

        if (project_id or project_name):
            _project = dict()

            if project_id:
                _project['id'] = project_id
            elif project_name:
                _project['name'] = project_name

            if project_domain_id is not None:
                _project['domain'] = {'id': project_domain_id}
            elif project_domain_name is not None:
                _project['domain'] = {'name': project_domain_name}

            creds['auth']['scope'] = dict(project=_project)
        elif domain_id:
            creds['auth']['scope'] = dict(domain={'id': domain_id})
        elif domain_name:
            creds['auth']['scope'] = dict(domain={'name': domain_name})
        elif system:
            creds['auth']['scope'] = dict(system={system: True})

        body = json.dumps(creds, sort_keys=True)
        resp, body = self.post(self.auth_url, body=body)
        self.expected_success(201, resp.status)
        return rest_client.ResponseBody(resp, body)

    def request(self, method, url, extra_headers=False, headers=None,
                body=None, chunked=False):
        """A simple HTTP request interface.

        Note: this overloads the `request` method from the parent class and
        thus must implement the same method signature.
        """
        if headers is None:
            # Always accept 'json', for xml token client too.
            # Because XML response is not easily
            # converted to the corresponding JSON one
            headers = self.get_headers(accept_type="json")
        elif extra_headers:
            try:
                headers.update(self.get_headers(accept_type="json"))
            except (ValueError, TypeError):
                headers = self.get_headers(accept_type="json")

        resp, resp_body = self.raw_request(url, method,
                                           headers=headers, body=body,
                                           log_req_body='<omitted>')
        if resp.status in [401, 403]:
            resp_body = json.loads(resp_body)
            raise exceptions.Unauthorized(resp_body['error']['message'])
        elif resp.status not in [200, 201, 204]:
            raise exceptions.IdentityError(
                'Unexpected status code {0}'.format(resp.status))

        return resp, json.loads(resp_body)

    def get_token(self, **kwargs):
        """Returns (token id, token data) for supplied credentials"""

        auth_data = kwargs.pop('auth_data', False)

        if not (kwargs.get('user_domain_id') or
                kwargs.get('user_domain_name')):
            kwargs['user_domain_name'] = 'Default'

        if not (kwargs.get('project_domain_id') or
                kwargs.get('project_domain_name')):
            kwargs['project_domain_name'] = 'Default'

        body = self.auth(**kwargs)

        token = body.response.get('x-subject-token')
        if auth_data:
            return token, body['token']
        else:
            return token


class V3TokenClientJSON(V3TokenClient):
    LOG = logging.getLogger(__name__)

    def _warn(self):
        self.LOG.warning("%s class was deprecated and renamed to %s",
                         self.__class__.__name__, 'V3TokenClient')

    def __init__(self, *args, **kwargs):
        self._warn()
        super(V3TokenClientJSON, self).__init__(*args, **kwargs)



# Copyright 2015 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.services.network import base


class SubnetpoolsClient(base.BaseNetworkClient):

    def list_subnetpools(self, **filters):
        """Lists subnet pools to which the tenant has access.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/network/v2/index.html#list-subnet-pools
        """
        uri = '/subnetpools'
        return self.list_resources(uri, **filters)

    def create_subnetpool(self, **kwargs):
        """Creates a subnet pool.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/network/v2/index.html#create-subnet-pool
        """
        uri = '/subnetpools'
        post_data = {'subnetpool': kwargs}
        return self.create_resource(uri, post_data)

    def show_subnetpool(self, subnetpool_id, **fields):
        """Shows information for a subnet pool.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/network/v2/index.html#show-subnet-pool
        """
        uri = '/subnetpools/%s' % subnetpool_id
        return self.show_resource(uri, **fields)

    def update_subnetpool(self, subnetpool_id, **kwargs):
        """Updates a subnet pool.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/network/v2/index.html#update-subnet-pool
        """
        uri = '/subnetpools/%s' % subnetpool_id
        post_data = {'subnetpool': kwargs}
        return self.update_resource(uri, post_data)

    def delete_subnetpool(self, subnetpool_id):
        uri = '/subnetpools/%s' % subnetpool_id
        return self.delete_resource(uri)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import ast
import importlib
import os
import shutil
import sys
import tempfile
from unittest import mock

from tempest.lib.cmd import check_uuid
from tempest.lib import decorators
from tempest.tests import base


class TestCLInterface(base.TestCase):
    CODE = "import unittest\n" \
           "class TestClass(unittest.TestCase):\n" \
           "    def test_tests(self):\n" \
           "        pass"

    def create_tests_file(self, directory):
        with open(directory + "/__init__.py", "w"):
            pass

        tests_file = directory + "/tests.py"
        with open(tests_file, "w") as fake_file:
            fake_file.write(TestCLInterface.CODE)

        return tests_file

    def test_fix_argument_no(self):
        temp_dir = tempfile.mkdtemp(prefix='check-uuid-no', dir=".")
        self.addCleanup(shutil.rmtree, temp_dir, ignore_errors=True)
        tests_file = self.create_tests_file(temp_dir)
        sys.argv = [sys.argv[0]] + ["--package",
                                    os.path.relpath(temp_dir)]

        self.assertRaises(SystemExit, check_uuid.run)
        with open(tests_file, "r") as f:
            self.assertTrue(TestCLInterface.CODE == f.read())

    @decorators.skip_because(bug='1918316')
    def test_fix_argument_yes(self):
        temp_dir = tempfile.mkdtemp(prefix='check-uuid-yes', dir=".")
        self.addCleanup(shutil.rmtree, temp_dir, ignore_errors=True)
        tests_file = self.create_tests_file(temp_dir)

        sys.argv = [sys.argv[0]] + ["--fix", "--package",
                                    os.path.relpath(temp_dir)]
        check_uuid.run()
        with open(tests_file, "r") as f:
            self.assertTrue(TestCLInterface.CODE != f.read())


class TestSourcePatcher(base.TestCase):
    def test_add_patch(self):
        patcher = check_uuid.SourcePatcher()
        fake_file = tempfile.NamedTemporaryFile("w+t", delete=False)
        file_contents = 'first_line\nsecond_line'
        fake_file.write(file_contents)
        fake_file.close()
        patcher.add_patch(fake_file.name, 'patch', 2)

        source_file = patcher.source_files[fake_file.name]
        self.assertEqual(1, len(patcher.patches))
        (patch_id, patch), = patcher.patches.items()
        self.assertEqual(patcher._quote('patch\n'), patch)
        self.assertEqual('first_line\n{%s:s}second_line' % patch_id,
                         patcher._unquote(source_file))

    def test_apply_patches(self):
        fake_file = tempfile.NamedTemporaryFile("w+t")
        patcher = check_uuid.SourcePatcher()
        patcher.patches = {'fake-uuid': patcher._quote('patch\n')}
        patcher.source_files = {
            fake_file.name: patcher._quote('first_line\n') +
            '{fake-uuid:s}second_line'}
        with mock.patch('sys.stdout'):
            patcher.apply_patches()

        lines = fake_file.read().split('\n')
        fake_file.close()
        self.assertEqual(['first_line', 'patch', 'second_line'], lines)
        self.assertFalse(patcher.patches)
        self.assertFalse(patcher.source_files)


class TestTestChecker(base.TestCase):
    IMPORT_LINE = "from tempest.lib import decorators\n"

    def _test_add_uuid_to_test(self, source_file):
        class Fake_test_node():
            lineno = 1
            col_offset = 4
        patcher = check_uuid.SourcePatcher()
        checker = check_uuid.TestChecker(importlib.import_module('tempest'))
        fake_file = tempfile.NamedTemporaryFile("w+t", delete=False)
        fake_file.write(source_file)
        fake_file.close()
        checker._add_uuid_to_test(patcher, Fake_test_node(), fake_file.name)

        self.assertEqual(1, len(patcher.patches))
        self.assertEqual(1, len(patcher.source_files))
        (patch_id, patch), = patcher.patches.items()
        changed_source_file, = patcher.source_files.values()
        self.assertEqual('{%s:s}%s' % (patch_id, patcher._quote(source_file)),
                         changed_source_file)
        expected_patch_start = patcher._quote(
            '    ' + check_uuid.DECORATOR_TEMPLATE.split('(')[0])
        self.assertTrue(patch.startswith(expected_patch_start))

    def test_add_uuid_to_test_def(self):
        source_file = ("    def test_test():\n"
                       "        pass")
        self._test_add_uuid_to_test(source_file)

    def test_add_uuid_to_test_decorator(self):
        source_file = ("    @decorators.idempotent_id\n"
                       "    def test_test():\n"
                       "        pass")
        self._test_add_uuid_to_test(source_file)

    @staticmethod
    def get_mocked_ast_object(lineno, col_offset, module, name, object_type):
        ast_object = mock.Mock(spec=object_type)
        name_obj = mock.Mock()
        ast_object.lineno = lineno
        ast_object.col_offset = col_offset
        name_obj.name = name
        ast_object.module = module
        ast_object.names = [name_obj]

        return ast_object

    def test_add_import_for_test_uuid_no_tempest(self):
        patcher = check_uuid.SourcePatcher()
        checker = check_uuid.TestChecker(importlib.import_module('tempest'))
        fake_file = tempfile.NamedTemporaryFile("w+t", delete=False)
        source_code = "from unittest import mock\n"
        fake_file.write(source_code)
        fake_file.close()

        class Fake_src_parsed():
            body = [TestTestChecker.get_mocked_ast_object(
                1, 4, 'unittest', 'mock', ast.ImportFrom)]

        checker._add_import_for_test_uuid(patcher, Fake_src_parsed,
                                          fake_file.name)
        patcher.apply_patches()

        with open(fake_file.name, "r") as f:
            expected_result = source_code + '\n' + TestTestChecker.IMPORT_LINE
            self.assertTrue(expected_result == f.read())

    def test_add_import_for_test_uuid_tempest(self):
        patcher = check_uuid.SourcePatcher()
        checker = check_uuid.TestChecker(importlib.import_module('tempest'))
        fake_file = tempfile.NamedTemporaryFile("w+t", delete=False)
        source_code = "from tempest import a_fake_module\n"
        fake_file.write(source_code)
        fake_file.close()

        class Fake_src_parsed:
            body = [TestTestChecker.get_mocked_ast_object(
                1, 4, 'tempest', 'a_fake_module', ast.ImportFrom)]

        checker._add_import_for_test_uuid(patcher, Fake_src_parsed,
                                          fake_file.name)
        patcher.apply_patches()

        with open(fake_file.name, "r") as f:
            expected_result = source_code + TestTestChecker.IMPORT_LINE
            self.assertTrue(expected_result == f.read())

    def test_add_import_no_import(self):
        patcher = check_uuid.SourcePatcher()
        patcher.add_patch = mock.Mock()
        checker = check_uuid.TestChecker(importlib.import_module('tempest'))
        fake_file = tempfile.NamedTemporaryFile("w+t", delete=False)
        fake_file.close()

        class Fake_src_parsed:
            body = []

        checker._add_import_for_test_uuid(patcher, Fake_src_parsed,
                                          fake_file.name)

        self.assertTrue(not patcher.add_patch.called)



# Copyright 2013 IBM Corp.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_serialization import jsonutils as json

from tempest.lib.common import rest_client


class ResourceTypesClient(rest_client.RestClient):
    api_version = "v2"

    def list_resource_types(self):
        """Lists all resource types.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/image/v2/metadefs-index.html#list-resource-types
        """
        url = 'metadefs/resource_types'
        resp, body = self.get(url)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def create_resource_type_association(self, namespace_id, **kwargs):
        """Creates a resource type association in given namespace.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/image/v2/metadefs-index.html#create-resource-type-association
        """
        url = 'metadefs/namespaces/%s/resource_types' % namespace_id
        data = json.dumps(kwargs)
        resp, body = self.post(url, data)
        self.expected_success(201, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def list_resource_type_association(self, namespace_id):
        """Lists resource type associations in given namespace.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/image/v2/metadefs-index.html#list-resource-type-associations
        """
        url = 'metadefs/namespaces/%s/resource_types' % namespace_id
        resp, body = self.get(url)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def delete_resource_type_association(self, namespace_id, resource_name):
        """Removes resource type association in given namespace.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/image/v2/metadefs-index.html#remove-resource-type-association
        """
        url = 'metadefs/namespaces/%s/resource_types/%s' % (namespace_id,
                                                            resource_name)
        resp, _ = self.delete(url)
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp)



# Copyright 2016 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import operator

import testtools

from tempest.api.volume import base
from tempest import config
from tempest.lib import decorators

CONF = config.CONF


class VolumesDeleteCascade(base.BaseVolumeTest):
    """Delete a volume with associated snapshots.

    Cinder provides the ability to delete a volume with its
    associated snapshots.
    It is allow a volume and its snapshots to be removed in one operation
    both for usability and performance reasons.
    """

    @classmethod
    def skip_checks(cls):
        super(VolumesDeleteCascade, cls).skip_checks()
        if not CONF.volume_feature_enabled.snapshot:
            raise cls.skipException("Cinder snapshot feature disabled")

    def _assert_cascade_delete(self, volume_id):
        # Fetch volume ids
        volume_list = [
            vol['id'] for vol in
            self.volumes_client.list_volumes()['volumes']
            ]

        # Verify the parent volume was deleted
        self.assertNotIn(volume_id, volume_list)

        # List snapshots
        snapshot_list = self.snapshots_client.list_snapshots()['snapshots']

        # Verify snapshots were deleted
        self.assertNotIn(volume_id, map(operator.itemgetter('volume_id'),
                                        snapshot_list))

    @decorators.idempotent_id('994e2d40-de37-46e8-b328-a58fba7e4a95')
    def test_volume_delete_cascade(self):
        """Test deleting a volume with associated snapshots

        The case validates the ability to delete a volume
        with associated snapshots.
        """

        # Create a volume
        volume = self.create_volume()

        for _ in range(2):
            self.create_snapshot(volume['id'])

        # Delete the parent volume with associated snapshots
        self.volumes_client.delete_volume(volume['id'], cascade=True)
        self.volumes_client.wait_for_resource_deletion(volume['id'])

        # Verify volume parent was deleted with its associated snapshots
        self._assert_cascade_delete(volume['id'])

    @decorators.idempotent_id('59a77ede-609b-4ee8-9f68-fc3c6ffe97b5')
    @testtools.skipIf(CONF.volume.storage_protocol == 'ceph',
                      'Skip because of Bug#1677525')
    def test_volume_from_snapshot_cascade_delete(self):
        """Test deleting a volume with associated volume-associated snapshot

        The case validates the ability to delete a volume with
        associated snapshot while there is another volume created
        from that snapshot.
        """

        # Create a volume
        volume = self.create_volume()

        snapshot = self.create_snapshot(volume['id'])

        # Create volume from snapshot
        volume_snap = self.create_volume(snapshot_id=snapshot['id'])
        volume_details = self.volumes_client.show_volume(
            volume_snap['id'])['volume']
        self.assertEqual(snapshot['id'], volume_details['snapshot_id'])

        # Delete the parent volume with associated snapshot
        self.volumes_client.delete_volume(volume['id'], cascade=True)
        self.volumes_client.wait_for_resource_deletion(volume['id'])

        # Verify volume parent was deleted with its associated snapshot
        self._assert_cascade_delete(volume['id'])



# Copyright 2015 NEC Corporation. All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.compute import base
from tempest.common import utils
from tempest.lib import decorators


class ComputeTenantNetworksTest(base.BaseV2ComputeTest):
    """Test compute tenant networks API with microversion less than 2.36"""

    max_microversion = '2.35'

    @classmethod
    def resource_setup(cls):
        super(ComputeTenantNetworksTest, cls).resource_setup()
        cls.client = cls.os_primary.tenant_networks_client
        cls.network = cls.get_tenant_network()

    @classmethod
    def setup_credentials(cls):
        cls.set_network_resources(network=True)
        super(ComputeTenantNetworksTest, cls).setup_credentials()

    @decorators.idempotent_id('edfea98e-bbe3-4c7a-9739-87b986baff26')
    @utils.services('network')
    def test_list_show_tenant_networks(self):
        """Test list/show tenant networks

        Fetch all networks that are visible to the tenant: this may include
        shared and external networks.
        """
        tenant_networks = [
            n['id'] for n in self.client.list_tenant_networks()['networks']
        ]
        self.assertIn(self.network['id'], tenant_networks,
                      "No tenant networks found.")

        net = self.client.show_tenant_network(self.network['id'])
        self.assertEqual(self.network['id'], net['network']['id'])



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_serialization import jsonutils as json

from tempest.lib.common import rest_client
from tempest.lib import exceptions as lib_exc


class EncryptionTypesClient(rest_client.RestClient):

    def is_resource_deleted(self, id):
        try:
            body = self.show_encryption_type(id)
            if not body:
                return True
        except lib_exc.NotFound:
            return True
        return False

    @property
    def resource_type(self):
        """Returns the primary type of resource this client works with."""
        return 'encryption-type'

    def show_encryption_type(self, volume_type_id):
        """Get the volume encryption type for the specified volume type.

        :param volume_type_id: Id of volume type.
        """
        url = "/types/%s/encryption" % volume_type_id
        resp, body = self.get(url)
        body = json.loads(body)
        self.expected_success(200, resp.status)
        return rest_client.ResponseBody(resp, body)

    def create_encryption_type(self, volume_type_id, **kwargs):
        """Create encryption type.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/block-storage/v2/#create-an-encryption-type-for-v2
        """
        url = "/types/%s/encryption" % volume_type_id
        post_body = json.dumps({'encryption': kwargs})
        resp, body = self.post(url, post_body)
        body = json.loads(body)
        self.expected_success(200, resp.status)
        return rest_client.ResponseBody(resp, body)

    def delete_encryption_type(self, volume_type_id):
        """Delete the encryption type for the specified volume type."""
        resp, body = self.delete(
            "/types/%s/encryption/provider" % volume_type_id)
        self.expected_success(202, resp.status)
        return rest_client.ResponseBody(resp, body)



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from urllib import parse as urllib

from oslo_serialization import jsonutils as json

from tempest.lib.api_schema.response.compute.v2_1 import flavors as schema
from tempest.lib.api_schema.response.compute.v2_1 import flavors_access \
    as schema_access
from tempest.lib.api_schema.response.compute.v2_1 import flavors_extra_specs \
    as schema_extra_specs
from tempest.lib.api_schema.response.compute.v2_55 import flavors \
    as schemav255
from tempest.lib.api_schema.response.compute.v2_61 import flavors \
    as schemav261
from tempest.lib.common import rest_client
from tempest.lib.services.compute import base_compute_client


class FlavorsClient(base_compute_client.BaseComputeClient):

    schema_versions_info = [
        {'min': None, 'max': '2.54', 'schema': schema},
        {'min': '2.55', 'max': '2.60', 'schema': schemav255},
        {'min': '2.61', 'max': None, 'schema': schemav261}]

    def list_flavors(self, detail=False, **params):
        """Lists flavors.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#list-flavors
        https://docs.openstack.org/api-ref/compute/#list-flavors-with-details
        """
        url = 'flavors'
        schema = self.get_schema(self.schema_versions_info)
        if detail:
            url += '/detail'
            _schema = schema.list_flavors_details
        else:
            _schema = schema.list_flavors
        if params:
            url += '?%s' % urllib.urlencode(params)

        resp, body = self.get(url)
        body = json.loads(body)
        self.validate_response(_schema, resp, body)
        return rest_client.ResponseBody(resp, body)

    def show_flavor(self, flavor_id):
        """Shows details for a flavor.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#show-flavor-details
        """
        resp, body = self.get("flavors/%s" % flavor_id)
        body = json.loads(body)
        schema = self.get_schema(self.schema_versions_info)
        self.validate_response(schema.create_update_get_flavor_details,
                               resp, body)
        return rest_client.ResponseBody(resp, body)

    def create_flavor(self, **kwargs):
        """Create a new flavor or instance type.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#create-flavor
        """
        if 'ephemeral' in kwargs:
            kwargs['OS-FLV-EXT-DATA:ephemeral'] = kwargs.pop('ephemeral')
        if 'is_public' in kwargs:
            kwargs['os-flavor-access:is_public'] = kwargs.pop('is_public')

        post_body = json.dumps({'flavor': kwargs})
        resp, body = self.post('flavors', post_body)

        body = json.loads(body)
        schema = self.get_schema(self.schema_versions_info)
        self.validate_response(schema.create_update_get_flavor_details,
                               resp, body)
        return rest_client.ResponseBody(resp, body)

    def update_flavor(self, flavor_id, **kwargs):
        """Uodate the flavor or instance type.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#update-flavor-description
        """
        put_body = json.dumps({'flavor': kwargs})
        resp, body = self.put("flavors/%s" % flavor_id, put_body)

        body = json.loads(body)
        schema = self.get_schema(self.schema_versions_info)
        self.validate_response(schema.create_update_get_flavor_details,
                               resp, body)
        return rest_client.ResponseBody(resp, body)

    def delete_flavor(self, flavor_id):
        """Delete the given flavor.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#delete-flavor
        """
        resp, body = self.delete("flavors/{0}".format(flavor_id))
        self.validate_response(schema.delete_flavor, resp, body)
        return rest_client.ResponseBody(resp, body)

    def is_resource_deleted(self, id):
        # Did not use show_flavor(id) for verification as it gives
        # 200 ok even for deleted id. LP #981263
        # we can remove the loop here and use get by ID when bug gets sortedout
        flavors = self.list_flavors(detail=True)['flavors']
        for flavor in flavors:
            if flavor['id'] == id:
                return False
        return True

    @property
    def resource_type(self):
        """Return the primary type of resource this client works with."""
        return 'flavor'

    def set_flavor_extra_spec(self, flavor_id, **kwargs):
        """Set extra Specs to the mentioned flavor.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#create-extra-specs-for-a-flavor
        """
        post_body = json.dumps({'extra_specs': kwargs})
        resp, body = self.post('flavors/%s/os-extra_specs' % flavor_id,
                               post_body)
        body = json.loads(body)
        self.validate_response(schema_extra_specs.set_get_flavor_extra_specs,
                               resp, body)
        return rest_client.ResponseBody(resp, body)

    def list_flavor_extra_specs(self, flavor_id):
        """Get extra Specs details of the mentioned flavor.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#list-extra-specs-for-a-flavor
        """
        resp, body = self.get('flavors/%s/os-extra_specs' % flavor_id)
        body = json.loads(body)
        self.validate_response(schema_extra_specs.set_get_flavor_extra_specs,
                               resp, body)
        return rest_client.ResponseBody(resp, body)

    def show_flavor_extra_spec(self, flavor_id, key):
        """Get extra Specs key-value of the mentioned flavor and key.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#show-an-extra-spec-for-a-flavor
        """
        resp, body = self.get('flavors/%s/os-extra_specs/%s' % (flavor_id,
                                                                key))
        body = json.loads(body)
        self.validate_response(
            schema_extra_specs.set_get_flavor_extra_specs_key,
            resp, body)
        return rest_client.ResponseBody(resp, body)

    def update_flavor_extra_spec(self, flavor_id, key, **kwargs):
        """Update specified extra Specs of the mentioned flavor and key.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#update-an-extra-spec-for-a-flavor
        """
        resp, body = self.put('flavors/%s/os-extra_specs/%s' %
                              (flavor_id, key), json.dumps(kwargs))
        body = json.loads(body)
        self.validate_response(
            schema_extra_specs.set_get_flavor_extra_specs_key,
            resp, body)
        return rest_client.ResponseBody(resp, body)

    def unset_flavor_extra_spec(self, flavor_id, key):  # noqa
        # NOTE: This noqa is for passing T111 check and we cannot rename
        #       to keep backwards compatibility.
        """Unset extra Specs from the mentioned flavor.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#delete-an-extra-spec-for-a-flavor
        """
        resp, body = self.delete('flavors/%s/os-extra_specs/%s' %
                                 (flavor_id, key))
        self.validate_response(schema_extra_specs.unset_flavor_extra_specs,
                               resp, body)
        return rest_client.ResponseBody(resp, body)

    def list_flavor_access(self, flavor_id):
        """Get flavor access information given the flavor id.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#list-flavor-access-information-for-given-flavor
        """
        resp, body = self.get('flavors/%s/os-flavor-access' % flavor_id)
        body = json.loads(body)
        self.validate_response(schema_access.add_remove_list_flavor_access,
                               resp, body)
        return rest_client.ResponseBody(resp, body)

    def add_flavor_access(self, flavor_id, tenant_id):
        """Add flavor access for the specified tenant.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#add-flavor-access-to-tenant-addtenantaccess-action
        """
        post_body = {
            'addTenantAccess': {
                'tenant': tenant_id
            }
        }
        post_body = json.dumps(post_body)
        resp, body = self.post('flavors/%s/action' % flavor_id, post_body)
        body = json.loads(body)
        self.validate_response(schema_access.add_remove_list_flavor_access,
                               resp, body)
        return rest_client.ResponseBody(resp, body)

    def remove_flavor_access(self, flavor_id, tenant_id):
        """Remove flavor access from the specified tenant.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#remove-flavor-access-from-tenant-removetenantaccess-action
        """
        post_body = {
            'removeTenantAccess': {
                'tenant': tenant_id
            }
        }
        post_body = json.dumps(post_body)
        resp, body = self.post('flavors/%s/action' % flavor_id, post_body)
        body = json.loads(body)
        self.validate_response(schema_access.add_remove_list_flavor_access,
                               resp, body)
        return rest_client.ResponseBody(resp, body)



# Copyright (c) 2016 Hewlett-Packard Enterprise Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy of
# the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations under
# the License.

from tempest.lib.services.network.agents_client import AgentsClient
from tempest.lib.services.network.extensions_client import ExtensionsClient
from tempest.lib.services.network.floating_ips_client import FloatingIPsClient
from tempest.lib.services.network.metering_label_rules_client import \
    MeteringLabelRulesClient
from tempest.lib.services.network.metering_labels_client import \
    MeteringLabelsClient
from tempest.lib.services.network.networks_client import NetworksClient
from tempest.lib.services.network.ports_client import PortsClient
from tempest.lib.services.network.qos_client import QosClient
from tempest.lib.services.network.qos_minimum_bandwidth_rules_client import \
    QosMinimumBandwidthRulesClient
from tempest.lib.services.network.quotas_client import QuotasClient
from tempest.lib.services.network.routers_client import RoutersClient
from tempest.lib.services.network.security_group_rules_client import \
    SecurityGroupRulesClient
from tempest.lib.services.network.security_groups_client import \
    SecurityGroupsClient
from tempest.lib.services.network.segments_client import SegmentsClient
from tempest.lib.services.network.service_providers_client import \
    ServiceProvidersClient
from tempest.lib.services.network.subnetpools_client import SubnetpoolsClient
from tempest.lib.services.network.subnets_client import SubnetsClient
from tempest.lib.services.network.tags_client import TagsClient
from tempest.lib.services.network.versions_client import NetworkVersionsClient

__all__ = ['AgentsClient', 'ExtensionsClient', 'FloatingIPsClient',
           'MeteringLabelRulesClient', 'MeteringLabelsClient',
           'NetworksClient', 'NetworkVersionsClient', 'PortsClient',
           'QosClient', 'QosMinimumBandwidthRulesClient', 'QuotasClient',
           'RoutersClient', 'SecurityGroupRulesClient', 'SecurityGroupsClient',
           'SegmentsClient', 'ServiceProvidersClient', 'SubnetpoolsClient',
           'SubnetsClient', 'TagsClient']
